#!/usr/bin/env python
# -*- coding: utf-8 -*-

# dialogs.database.py
"""
Classes based on wx.Dialog for models.database_editor.py
In general, this classes will manifest themselves in the GUI on
initialization. They should contain a function names
action which will be executed on a dialog resolution of wx.ID_OK
"""
# Copyright (c) 2016-2019 Dan Cutright
# This file is part of DVH Analytics, released under a BSD license.
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics

import wx
from datetime import datetime
from os import mkdir, rename
from os.path import join, basename
from dvha.db.sql_connector import (
    DVH_SQL,
    echo_sql_db,
    is_file_sqlite_db,
    write_test,
)
import dvha.db.update as db_update
from dvha.models.import_dicom import ImportDicomFrame
from dvha.paths import DATA_DIR
from dvha.tools.errors import SQLError, SQLErrorDialog
from dvha.tools.utilities import (
    delete_directory_contents,
    move_files_to_new_path,
    delete_file,
    get_file_paths,
    delete_imported_dicom_files,
    move_imported_dicom_files,
    MessageDialog,
    get_window_size,
)
from dvha.tools.threading_progress import ProgressFrame
from pubsub import pub


class CalculationsDialog(wx.Dialog):
    """
    Dialog to perform various calcs for values not stored in DICOM files.
    """

    def __init__(self):
        wx.Dialog.__init__(self, None, title="Calculations")

        self.choices = [
            "PTV Distances",
            "PTV Overlap",
            "ROI Spread",
            "ROI Cross-Section",
            "ROI Surface Area",
            "OAR-PTV Centroid Distance",
            "All",
        ]
        self.combo_box_calculate = wx.ComboBox(
            self,
            wx.ID_ANY,
            choices=self.choices,
            style=wx.CB_DROPDOWN | wx.CB_READONLY,
        )
        self.text_ctrl_condition = wx.TextCtrl(self, wx.ID_ANY, "")
        self.button_ok = wx.Button(self, wx.ID_OK, "Calculate")
        self.button_cancel = wx.Button(self, wx.ID_CANCEL, "Cancel")

        self.__do_layout()

        self.LUT = {
            "PTV Distances": {
                "func": db_update.update_ptv_dist_data,
                "title": "Calculating PTV Distance Metrics",
            },
            "PTV Overlap": {
                "func": db_update.update_ptv_overlap,
                "title": "Calculating PTV Overlap",
            },
            "OAR-PTV Centroid Distance": {
                "func": db_update.update_ptv_centroid_distances,
                "title": "Calculating OAR-PTV Centroid Distances",
            },
            "ROI Spread": {
                "func": db_update.update_roi_spread,
                "title": "Calculating ROI Spreads",
            },
            "ROI Cross-Section": {
                "func": db_update.update_roi_cross_section,
                "title": "Calculating ROI Cross-Sections",
            },
            "ROI Surface Area": {
                "func": db_update.update_roi_surface_area,
                "title": "Calculating ROI Surface Areas",
            },
        }

        pub.subscribe(self.do_next_calculation, "do_next_calculation")

        self.close_msg = None

        self.run()

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_wrapper_inner = wx.BoxSizer(wx.VERTICAL)
        sizer_ok_cancel = wx.BoxSizer(wx.HORIZONTAL)
        sizer_input = wx.StaticBoxSizer(
            wx.StaticBox(self, wx.ID_ANY, ""), wx.VERTICAL
        )
        sizer_condition = wx.BoxSizer(wx.VERTICAL)
        sizer_calc_and_check = wx.BoxSizer(wx.HORIZONTAL)
        sizer_calculate = wx.BoxSizer(wx.VERTICAL)

        label_calculate = wx.StaticText(self, wx.ID_ANY, "Calculate:")
        sizer_calculate.Add(label_calculate, 0, wx.BOTTOM, 5)
        sizer_calculate.Add(self.combo_box_calculate, 0, 0, 0)
        sizer_calc_and_check.Add(sizer_calculate, 0, wx.EXPAND, 0)
        sizer_input.Add(
            sizer_calc_and_check,
            0,
            wx.BOTTOM | wx.EXPAND | wx.LEFT | wx.RIGHT,
            5,
        )

        label_condition = wx.StaticText(self, wx.ID_ANY, "Condition:")
        sizer_condition.Add(label_condition, 0, wx.BOTTOM, 5)
        sizer_condition.Add(self.text_ctrl_condition, 0, wx.EXPAND, 0)
        sizer_input.Add(sizer_condition, 0, wx.ALL | wx.EXPAND, 5)
        sizer_wrapper_inner.Add(sizer_input, 0, wx.ALL | wx.EXPAND, 5)

        sizer_ok_cancel.Add(self.button_ok, 0, wx.ALL, 5)
        sizer_ok_cancel.Add(self.button_cancel, 0, wx.ALL, 5)
        sizer_wrapper_inner.Add(sizer_ok_cancel, 0, wx.ALIGN_RIGHT | wx.ALL, 5)

        sizer_wrapper.Add(sizer_wrapper_inner, 0, wx.EXPAND, 0)

        self.SetSizer(sizer_wrapper)
        sizer_wrapper.Fit(self)
        self.Layout()

        self.SetSize((get_window_size(0.3, 1)[0], self.Size[1]))
        self.Center()

    def run(self):
        res = self.ShowModal()
        if res == wx.ID_OK:
            if self.calculation == "All":
                self.close_msg = "do_next_calculation"
                self.do_next_calculation(init=True)
            else:
                self.run_calculation()
                self.close()

    def close(self):
        pub.unsubAll(topicName="do_next_calculation")
        self.Destroy()

    def run_calculation(self):
        ProgressFrame(
            self.threading_obj_list,
            self.func,
            title=self.title,
            close_msg=self.close_msg,
            action_msg="Processing Study",
            sub_gauge=True,
            kwargs=True,
        )

    def do_next_calculation(self, init=False):
        if init:
            self.combo_box_calculate.SetValue(self.choices[0])

        if self.calculation != "All":
            self.run_calculation()
            index = self.choices.index(self.calculation)
            self.combo_box_calculate.SetValue(self.choices[index + 1])
        else:
            self.close()

    @property
    def condition(self):
        text = self.text_ctrl_condition.GetValue()
        return text if text else None

    @property
    def threading_obj_list(self):
        uids = db_update.query(
            "DVHs", "study_instance_uid", self.condition, unique=True
        )
        return [{"uid": uid, "callback": self.callback} for uid in uids]

    @property
    def calculation(self):
        return self.combo_box_calculate.GetValue()

    @property
    def title(self):
        return self.LUT[self.calculation]["title"]

    @property
    def func(self):
        return self.LUT[self.calculation]["func"]

    @staticmethod
    def callback(msg):
        pub.sendMessage("sub_progress_update", msg=msg)


class ChangeOrDeleteBaseClass(wx.Dialog):
    """
    A class generalized for changing a patient identifier or deleting a study for a patient
    """

    def __init__(
        self,
        text_input_1_label,
        text_input_2_label,
        ok_button_label,
        title,
        mrn=None,
        study_instance_uid=None,
    ):
        """
        :param text_input_1_label: label of first text input in GUI
        :type text_input_1_label: str
        :param text_input_2_label: label of second text input in GUI
        :type text_input_2_label: str
        :param ok_button_label: label of OK button in GUI
        :type ok_button_label: str
        :param title: title of the wx.Dialog object
        :type title: str
        :param mrn: optional initial value for mrn
        :type mrn: str
        :param study_instance_uid: options initial value for study instance uid
        :type study_instance_uid: str
        """
        wx.Dialog.__init__(self, None, title=title)

        self.initial_mrn = mrn
        self.initial_study_instance_uid = study_instance_uid

        self.text_input_1_label = text_input_1_label
        self.text_input_2_label = text_input_2_label

        self.combo_box_patient_identifier = wx.ComboBox(
            self,
            wx.ID_ANY,
            choices=["MRN", "Study Instance UID"],
            style=wx.CB_DROPDOWN | wx.CB_READONLY,
        )
        self.text_ctrl_1 = wx.TextCtrl(self, wx.ID_ANY, "")
        self.text_ctrl_2 = wx.TextCtrl(self, wx.ID_ANY, "")
        self.button_ok = wx.Button(self, wx.ID_OK, ok_button_label)
        self.button_ok.Disable()
        self.button_cancel = wx.Button(self, wx.ID_CANCEL, "Cancel")

        self.__set_properties()
        self.__do_bind()
        self.__do_layout()

        self.run()

    def __set_properties(self):
        self.text_ctrl_1.SetMinSize((365, 22))
        if self.initial_mrn or self.initial_study_instance_uid is None:
            self.combo_box_patient_identifier.SetValue("MRN")
        else:
            self.combo_box_patient_identifier.SetValue("Study Instance UID")
        self.on_identifier_change(None)

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_ok_cancel = wx.BoxSizer(wx.HORIZONTAL)
        sizer_input = wx.StaticBoxSizer(
            wx.StaticBox(self, wx.ID_ANY, ""), wx.VERTICAL
        )
        sizer_new_value = wx.BoxSizer(wx.VERTICAL)
        sizer_value = wx.BoxSizer(wx.VERTICAL)
        sizer_patient_identifier = wx.BoxSizer(wx.HORIZONTAL)
        label_patient_identifier = wx.StaticText(
            self, wx.ID_ANY, "Patient Identifier:"
        )
        sizer_patient_identifier.Add(label_patient_identifier, 0, wx.ALL, 5)
        sizer_patient_identifier.Add(
            self.combo_box_patient_identifier, 0, wx.TOP, 2
        )
        sizer_input.Add(
            sizer_patient_identifier, 0, wx.EXPAND | wx.LEFT | wx.RIGHT, 5
        )
        label_text_input_1 = wx.StaticText(
            self, wx.ID_ANY, self.text_input_1_label
        )
        sizer_value.Add(label_text_input_1, 0, wx.EXPAND | wx.ALL, 5)
        sizer_value.Add(
            self.text_ctrl_1, 0, wx.BOTTOM | wx.EXPAND | wx.LEFT | wx.RIGHT, 5
        )
        sizer_input.Add(sizer_value, 0, wx.EXPAND | wx.LEFT | wx.RIGHT, 5)
        label_text_input_2 = wx.StaticText(
            self, wx.ID_ANY, self.text_input_2_label
        )
        sizer_new_value.Add(label_text_input_2, 0, wx.EXPAND | wx.ALL, 5)
        sizer_new_value.Add(
            self.text_ctrl_2, 0, wx.BOTTOM | wx.EXPAND | wx.LEFT | wx.RIGHT, 5
        )
        sizer_input.Add(sizer_new_value, 0, wx.EXPAND | wx.LEFT | wx.RIGHT, 5)
        sizer_wrapper.Add(sizer_input, 0, wx.LEFT | wx.RIGHT | wx.TOP, 10)
        sizer_ok_cancel.Add(self.button_ok, 0, wx.ALL, 5)
        sizer_ok_cancel.Add(self.button_cancel, 0, wx.ALL, 5)
        sizer_wrapper.Add(
            sizer_ok_cancel, 0, wx.ALIGN_RIGHT | wx.BOTTOM | wx.RIGHT, 10
        )
        self.SetSizer(sizer_wrapper)
        sizer_wrapper.Fit(self)
        self.Layout()
        self.Center()

    def __do_bind(self):
        self.Bind(
            wx.EVT_COMBOBOX,
            self.on_identifier_change,
            id=self.combo_box_patient_identifier.GetId(),
        )
        self.Bind(wx.EVT_TEXT, self.text_ticker, id=self.text_ctrl_1.GetId())
        self.Bind(wx.EVT_TEXT, self.text_ticker, id=self.text_ctrl_2.GetId())

    def text_ticker(self, evt):
        [self.button_ok.Disable, self.button_ok.Enable][
            self.is_action_allowed
        ]()

    def run(self):
        res = self.ShowModal()
        if res == wx.ID_OK:
            self.action()
        self.Destroy()

    def action(self):
        """
        Should be over-written in classes that inherit this class, but needs to be defined here since run refers to it
        """
        pass

    def on_identifier_change(self, evt):
        value = {
            "Study Instance UID": self.initial_study_instance_uid,
            "MRN": self.initial_mrn,
        }[self.combo_box_patient_identifier.GetValue()]
        if value is not None:
            self.text_ctrl_1.SetValue(value)
            wx.CallAfter(self.text_ctrl_2.SetFocus)

    @property
    def sql_column(self):
        return (
            self.combo_box_patient_identifier.GetValue()
            .lower()
            .replace(" ", "_")
        )

    @property
    def is_id_valid(self):
        with DVH_SQL() as cnx:
            func = [cnx.is_uid_imported, cnx.is_mrn_imported][
                self.combo_box_patient_identifier.GetValue() == "MRN"
            ]
            ans = func(self.text_ctrl_1.GetValue())
        return ans

    @property
    def is_action_allowed(self):
        return self.is_id_valid and bool(self.text_ctrl_2.GetValue())


class ChangePatientIdentifierDialog(ChangeOrDeleteBaseClass):
    """
    Change MRN or Study Instance UID in all SQL tables
    """

    def __init__(self, mrn=None, study_instance_uid=None):
        """
        :param mrn: optional initial value for mrn
        :type mrn: str
        :param study_instance_uid: options initial value for study instance uid
        :type study_instance_uid: str
        """
        ChangeOrDeleteBaseClass.__init__(
            self,
            "Value:",
            "New Value:",
            "Change",
            "Change Patient Identifier",
            mrn=mrn,
            study_instance_uid=study_instance_uid,
        )

    def action(self):
        old_id = self.text_ctrl_1.GetValue()
        new_id = self.text_ctrl_2.GetValue()

        with DVH_SQL() as cnx:
            validation_func = [cnx.is_uid_imported, cnx.is_mrn_imported][
                self.sql_column == "mrn"
            ]
            change_func = [cnx.change_uid, cnx.change_mrn][
                self.sql_column == "mrn"
            ]

            if validation_func(old_id):
                if (
                    self.sql_column == "study_instance_uid"
                    and validation_func(new_id)
                ):
                    wx.MessageBox(
                        "This Study Instance UID is already in use.",
                        "%s Error"
                        % self.combo_box_patient_identifier.GetValue(),
                        wx.OK | wx.ICON_WARNING,
                    )
                else:
                    change_func(old_id, new_id)
            else:
                wx.MessageBox(
                    "No studies found with this %s."
                    % self.combo_box_patient_identifier.GetValue(),
                    "%s Error" % self.combo_box_patient_identifier.GetValue(),
                    wx.OK | wx.ICON_WARNING,
                )


class DeletePatientDialog(ChangeOrDeleteBaseClass):
    """
    Delete all data in SQL database for a given MRN or study instance uid
    """

    def __init__(self, mrn=None, study_instance_uid=None):
        """
        :param mrn: optional initial value for mrn
        :type mrn: str
        :param study_instance_uid: options initial value for study instance uid
        :type study_instance_uid: str
        """
        ChangeOrDeleteBaseClass.__init__(
            self,
            "Delete:",
            'Type "delete" to authorize:',
            "Delete",
            "Delete Patient",
            mrn=mrn,
            study_instance_uid=study_instance_uid,
        )

    def action(self):
        if self.text_ctrl_2.GetValue() == "delete":
            value = self.text_ctrl_1.GetValue()
            with DVH_SQL() as cnx:
                dicom_files = cnx.get_dicom_file_paths(
                    **{self.sql_column: value}
                )
                cnx.delete_rows("%s = '%s'" % (self.sql_column, value))

            DeleteFilesFromQuery(self, dicom_files)

    @property
    def is_action_allowed(self):
        return self.is_id_valid and self.text_ctrl_2.GetValue() == "delete"


class EditDatabaseDialog(wx.Dialog):
    """
    Generic UI object to edit values in SQL database with table, column, and condition
    """

    def __init__(self, inital_values=None):
        """
        :param inital_values: optional variable to provide initial values to table, column, value, or condition
        :type inital_values: dict
        """
        wx.Dialog.__init__(self, None, title="Edit Database Values")

        self.initial_values = inital_values
        self.error = False

        self.combo_box_table = wx.ComboBox(
            self,
            wx.ID_ANY,
            choices=self.tables,
            style=wx.CB_DROPDOWN | wx.CB_READONLY,
        )
        self.combo_box_column = wx.ComboBox(
            self, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN | wx.CB_READONLY
        )
        self.text_ctrl_value = wx.TextCtrl(self, wx.ID_ANY, "")
        self.text_ctrl_condition = wx.TextCtrl(self, wx.ID_ANY, "")
        self.button_ok = wx.Button(self, wx.ID_OK, "Update")
        self.button_ok.Disable()
        self.button_cancel = wx.Button(self, wx.ID_CANCEL, "Cancel")

        self.Bind(
            wx.EVT_COMBOBOX, self.table_ticker, id=self.combo_box_table.GetId()
        )
        self.Bind(
            wx.EVT_TEXT,
            self.condition_ticker,
            id=self.text_ctrl_condition.GetId(),
        )

        self.__set_properties()
        self.__do_layout()

        self.update_columns()

        self.run()

    def __set_properties(self):
        if self.initial_values:
            self.combo_box_table.SetValue(self.initial_values["table"])
            self.combo_box_column.SetValue(self.initial_values["column"])
            self.text_ctrl_value.SetValue(self.initial_values["value"])
            self.text_ctrl_condition.SetValue(self.initial_values["condition"])
        else:
            self.combo_box_table.SetValue("Plans")

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_wrapper_inner = wx.BoxSizer(wx.VERTICAL)
        sizer_ok_cancel = wx.BoxSizer(wx.HORIZONTAL)
        sizer_input = wx.StaticBoxSizer(
            wx.StaticBox(self, wx.ID_ANY, ""), wx.VERTICAL
        )
        sizer_condition = wx.BoxSizer(wx.VERTICAL)
        sizer_table_column_value = wx.BoxSizer(wx.HORIZONTAL)
        sizer_value = wx.BoxSizer(wx.VERTICAL)
        sizer_column = wx.BoxSizer(wx.VERTICAL)
        sizer_table = wx.BoxSizer(wx.VERTICAL)
        label_table = wx.StaticText(self, wx.ID_ANY, "Table:")
        sizer_table.Add(label_table, 0, 0, 0)
        sizer_table.Add(self.combo_box_table, 0, wx.EXPAND, 0)
        sizer_table_column_value.Add(sizer_table, 0, wx.ALL | wx.EXPAND, 5)
        label_column = wx.StaticText(self, wx.ID_ANY, "Column:")
        sizer_column.Add(label_column, 0, 0, 0)
        sizer_column.Add(self.combo_box_column, 0, 0, 0)
        sizer_table_column_value.Add(sizer_column, 0, wx.ALL | wx.EXPAND, 5)
        label_value = wx.StaticText(self, wx.ID_ANY, "Value:")
        sizer_value.Add(label_value, 0, 0, 0)
        sizer_value.Add(self.text_ctrl_value, 0, wx.EXPAND, 0)
        sizer_table_column_value.Add(sizer_value, 1, wx.ALL | wx.EXPAND, 5)
        sizer_input.Add(sizer_table_column_value, 1, wx.EXPAND, 0)
        label_condition = wx.StaticText(self, wx.ID_ANY, "Condition:")
        sizer_condition.Add(
            label_condition, 0, wx.BOTTOM | wx.RIGHT | wx.TOP, 5
        )
        sizer_condition.Add(self.text_ctrl_condition, 0, wx.EXPAND, 0)
        sizer_input.Add(
            sizer_condition, 1, wx.BOTTOM | wx.EXPAND | wx.LEFT | wx.RIGHT, 5
        )
        sizer_wrapper_inner.Add(
            sizer_input, 0, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.TOP, 5
        )
        sizer_ok_cancel.Add(self.button_ok, 0, wx.ALL, 5)
        sizer_ok_cancel.Add(self.button_cancel, 0, wx.ALL, 5)
        sizer_wrapper_inner.Add(sizer_ok_cancel, 0, wx.ALIGN_RIGHT | wx.ALL, 5)
        sizer_wrapper.Add(sizer_wrapper_inner, 0, wx.ALL | wx.EXPAND, 0)
        self.SetSizer(sizer_wrapper)
        sizer_wrapper.Fit(self)
        self.Layout()
        self.Center()

    @property
    def tables(self):
        with DVH_SQL() as cnx:
            tables = cnx.tables
        return tables

    @property
    def table(self):
        return self.combo_box_table.GetValue()

    @property
    def column(self):
        return self.combo_box_column.GetValue()

    @property
    def condition(self):
        return self.text_ctrl_condition.GetValue()

    @property
    def value(self):
        v = self.text_ctrl_value.GetValue()
        if v:
            return v
        return "NULL"

    def condition_ticker(self, evt):
        if self.condition:
            self.button_ok.Enable()
        else:
            self.button_ok.Disable()

    def update_columns(self):
        table = self.combo_box_table.GetValue()
        with DVH_SQL() as cnx:
            columns = cnx.get_column_names(table)
        self.combo_box_column.SetItems(columns)
        if self.combo_box_column.GetValue() not in columns:
            self.combo_box_column.SetValue(columns[0])

    def table_ticker(self, evt):
        self.update_columns()

    def run(self):
        res = self.ShowModal()
        if res == wx.ID_OK:
            self.update_db()
        self.Destroy()
        if self.error:
            EditDatabaseDialog(inital_values=self.values)

    def update_db(self):
        with DVH_SQL() as cnx:
            try:
                cnx.update(self.table, self.column, self.value, self.condition)
            except SQLError as sql_exception:
                SQLErrorDialog(self, sql_exception)
                self.error = True

    @property
    def values(self):
        return {
            "table": self.combo_box_table.GetValue(),
            "column": self.combo_box_column.GetValue(),
            "value": self.text_ctrl_value.GetValue(),
            "condition": self.text_ctrl_condition.GetValue(),
        }


class ReimportDialog(wx.Dialog):
    """
    Reimport data from catalogued DICOM files for a given MRN or study instance uid
    """

    def __init__(self, roi_map, options, mrn=None, study_instance_uid=None):
        """
        :param mrn: optional initial value for mrn
        :type mrn: str
        :param study_instance_uid: options initial value for study instance uid
        :type study_instance_uid: str
        """
        wx.Dialog.__init__(self, None, title="Reimport from DICOM")
        self.roi_map = roi_map
        self.options = options
        self.initial_mrn = mrn
        self.initial_uid = study_instance_uid

        self.text_ctrl_mrn = wx.TextCtrl(self, wx.ID_ANY, "")
        self.radio_box_delete_from_db = wx.RadioBox(
            self,
            wx.ID_ANY,
            "Current Data",
            choices=["Delete from DB", "Keep in DB"],
            majorDimension=2,
            style=wx.RA_SPECIFY_ROWS,
        )
        self.combo_box_study_date = wx.ComboBox(
            self, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN | wx.CB_READONLY
        )
        self.combo_box_uid = wx.ComboBox(
            self, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN | wx.CB_READONLY
        )
        self.button_reimport = wx.Button(self, wx.ID_OK, "Reimport")
        self.button_reimport.Disable()
        self.button_cancel = wx.Button(self, wx.ID_CANCEL, "Cancel")

        self.__set_properties()
        self.__do_layout()
        self.__do_bind()
        self.__apply_initial_values()

        self.run()

    def __set_properties(self):
        self.radio_box_delete_from_db.SetSelection(0)
        self.combo_box_study_date.SetMinSize((200, 25))

    def __do_bind(self):
        self.Bind(wx.EVT_TEXT, self.mrn_ticker, id=self.text_ctrl_mrn.GetId())
        self.Bind(
            wx.EVT_COMBOBOX,
            self.study_date_ticker,
            id=self.combo_box_study_date.GetId(),
        )
        self.Bind(
            wx.EVT_COMBOBOX, self.uid_ticker, id=self.combo_box_uid.GetId()
        )

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_ok_cancel = wx.BoxSizer(wx.HORIZONTAL)
        sizer_input = wx.StaticBoxSizer(
            wx.StaticBox(self, wx.ID_ANY, ""), wx.VERTICAL
        )
        sizer_input_date_uid = wx.BoxSizer(wx.HORIZONTAL)
        sizer_uid = wx.BoxSizer(wx.VERTICAL)
        sizer_input_date = wx.BoxSizer(wx.VERTICAL)
        sizer_input_mrn_db = wx.BoxSizer(wx.HORIZONTAL)
        sizer_mrn = wx.BoxSizer(wx.VERTICAL)
        label_mrn = wx.StaticText(self, wx.ID_ANY, "MRN:")
        sizer_mrn.Add(label_mrn, 0, wx.BOTTOM, 5)
        sizer_mrn.Add(self.text_ctrl_mrn, 0, wx.EXPAND | wx.RIGHT, 40)
        sizer_input_mrn_db.Add(sizer_mrn, 1, wx.TOP, 12)
        sizer_input_mrn_db.Add(self.radio_box_delete_from_db, 0, wx.ALL, 5)
        sizer_input.Add(sizer_input_mrn_db, 0, wx.EXPAND | wx.LEFT, 5)
        label_date = wx.StaticText(self, wx.ID_ANY, "Sim Study Date:")
        sizer_input_date.Add(label_date, 0, wx.BOTTOM, 5)
        sizer_input_date.Add(self.combo_box_study_date, 0, 0, 0)
        sizer_input_date_uid.Add(sizer_input_date, 0, wx.ALL | wx.EXPAND, 5)
        label_uid = wx.StaticText(self, wx.ID_ANY, "Study Instance UID:")
        sizer_uid.Add(label_uid, 0, wx.BOTTOM, 5)
        sizer_uid.Add(self.combo_box_uid, 0, wx.EXPAND, 0)
        sizer_input_date_uid.Add(sizer_uid, 1, wx.ALL | wx.EXPAND, 5)
        sizer_input.Add(sizer_input_date_uid, 0, wx.EXPAND, 0)
        sizer_wrapper.Add(sizer_input, 0, wx.ALL | wx.EXPAND, 5)
        sizer_ok_cancel.Add(self.button_reimport, 0, wx.ALL, 5)
        sizer_ok_cancel.Add(self.button_cancel, 0, wx.ALL, 5)
        sizer_wrapper.Add(
            sizer_ok_cancel, 0, wx.ALIGN_RIGHT | wx.BOTTOM | wx.RIGHT, 5
        )
        self.SetSizer(sizer_wrapper)
        sizer_wrapper.SetMinSize((700, 190))
        sizer_wrapper.Fit(self)
        self.Layout()
        self.Center()

    def __apply_initial_values(self):
        if self.initial_mrn is not None:
            self.text_ctrl_mrn.SetValue(self.initial_mrn)
        # if self.initial_uid is not None:
        #     self.combo_box_uid.SetSelection(self.initial_uid)

    def run(self):
        res = self.ShowModal()
        if res == wx.ID_OK:
            if self.uid:
                self.action()
        self.Destroy()

    def mrn_ticker(self, evt):
        with DVH_SQL() as cnx:
            is_mrn_valid = cnx.is_mrn_imported(self.mrn)
        if is_mrn_valid:
            self.update_study_dates()
        else:
            self.combo_box_study_date.SetItems([])
            self.combo_box_uid.SetItems([])
            self.button_reimport.Disable()

    def study_date_ticker(self, evt):
        self.update_uids()

    def uid_ticker(self, evt):
        if self.combo_box_uid.GetValue():
            self.button_reimport.Enable()
        else:
            self.button_reimport.Disable()

    def update_study_dates(self):
        with DVH_SQL() as cnx:
            choices = cnx.get_unique_values(
                "Plans", "sim_study_date", "mrn = '%s'" % self.mrn
            )
        self.combo_box_study_date.SetItems(choices)
        if choices:
            self.combo_box_study_date.SetValue(choices[0])
        self.update_uids()

    def update_uids(self):

        date = ["is NULL", "= '%s'::date" % self.sim_study_date][
            self.sim_study_date != "None"
        ]
        with DVH_SQL() as cnx:
            if cnx.db_type == "sqlite":
                date = date.replace("::date", "")
            condition = "mrn = '%s' and sim_study_date %s" % (self.mrn, date)
            choices = cnx.get_unique_values(
                "Plans", "study_instance_uid", condition
            )
        self.combo_box_uid.SetItems(choices)
        if choices:
            self.combo_box_uid.SetValue(choices[0])
        else:
            self.combo_box_uid.SetValue(None)
        self.uid_ticker(None)

    @property
    def delete_from_db(self):
        return self.radio_box_delete_from_db.GetSelection() == 0

    @property
    def mrn(self):
        return self.text_ctrl_mrn.GetValue()

    @property
    def sim_study_date(self):
        return self.combo_box_study_date.GetValue()

    @property
    def uid(self):
        return self.combo_box_uid.GetValue()

    def action(self):
        with DVH_SQL() as cnx:
            dicom_files = cnx.get_dicom_file_paths(uid=self.uid)
            if self.delete_from_db:
                cnx.delete_rows("study_instance_uid = '%s'" % self.uid)
        inbox = self.options.INBOX_DIR
        move_imported_dicom_files(dicom_files, inbox)
        ImportDicomFrame(
            self.roi_map, self.options, inbox=inbox, auto_parse=True
        )


class SQLSettingsDialog(wx.Dialog):
    """
    Edit and validate SQL connection settings
    """

    def __init__(self, options, group=1):
        wx.Dialog.__init__(self, None)

        self.options = options
        self.group = group

        self.keys = ["host", "port", "dbname", "user", "password"]

        self.input = {"host": wx.ComboBox(self, wx.ID_ANY, "")}
        for key in self.keys:
            if key not in {"password", "host"}:
                self.input[key] = wx.TextCtrl(self, wx.ID_ANY, "")
        self.input["password"] = wx.TextCtrl(
            self, wx.ID_ANY, "", style=wx.TE_PASSWORD
        )

        self.db_type_radiobox = wx.RadioBox(
            self, wx.ID_ANY, "Database Type", choices=["SQLite", "Postgres"]
        )

        self.button = {
            "echo": wx.Button(self, wx.ID_ANY, "Echo"),
            "write_test": wx.Button(self, wx.ID_ANY, "Write Test"),
            "reload": wx.Button(self, wx.ID_ANY, "Reload Last Connection"),
            "ok": wx.Button(self, wx.ID_OK, "OK"),
            "cancel": wx.Button(self, wx.ID_CANCEL, "Cancel"),
        }

        self.db_types = ["sqlite", "pgsql"]

        self.ip_history = self.options.SQL_PGSQL_IP_HIST

        self.Bind(
            wx.EVT_BUTTON, self.button_echo, id=self.button["echo"].GetId()
        )
        self.Bind(
            wx.EVT_BUTTON,
            self.button_write_test,
            id=self.button["write_test"].GetId(),
        )
        self.Bind(
            wx.EVT_BUTTON, self.button_reload, id=self.button["reload"].GetId()
        )
        self.Bind(
            wx.EVT_RADIOBOX, self.on_db_radio, id=self.db_type_radiobox.GetId()
        )

        if self.group == 2:
            self.sync_groups = wx.CheckBox(self, wx.ID_ANY, "Sync to Group 1")
            self.Bind(
                wx.EVT_CHECKBOX,
                self.on_sync_groups,
                id=self.sync_groups.GetId(),
            )

        self.__set_properties()
        self.__do_layout()
        self.Center()

        self.load_sql_settings()

        if self.group == 2:
            self.on_sync_groups()

        self.run()

    def __set_properties(self):
        prepend = ["", "Group 2: "][self.group != 1]
        self.SetTitle("%sSQL Connection Settings" % prepend)

        # Set initial db_type_radiobox to loaded settings or pgsql if none found
        self.set_selected_db_type(self.options.DB_TYPE_GRPS[self.group])

        self.button["reload"].Enable(self.has_last_cnx)

        if self.group == 2:
            self.sync_groups.SetValue(self.options.SYNC_SQL_CNX)

    def set_host_items(self):
        if self.selected_db_type == "sqlite":
            db_files = [
                basename(f)
                for f in get_file_paths(DATA_DIR, extension=".db")
                if is_file_sqlite_db(f)
            ]
            db_files.sort()
            self.input["host"].SetItems(db_files)
        else:
            self.input["host"].SetItems(self.ip_history)

    def __do_layout(self):
        sizer_frame = wx.BoxSizer(wx.VERTICAL)
        sizer_echo = wx.BoxSizer(wx.HORIZONTAL)
        sizer_write_test = wx.BoxSizer(wx.HORIZONTAL)
        sizer_reload = wx.BoxSizer(wx.HORIZONTAL)
        sizer_buttons = wx.BoxSizer(wx.HORIZONTAL)
        grid_sizer = wx.GridSizer(5, 2, 5, 10)

        label_titles = [
            "Host:",
            "Port:",
            "Database Name:",
            "User Name:",
            "Password:",
        ]
        self.label = {
            key: wx.StaticText(self, wx.ID_ANY, label_titles[i])
            for i, key in enumerate(self.keys)
        }

        for key in self.keys:
            grid_sizer.Add(self.label[key], 0, wx.ALL, 0)
            grid_sizer.Add(self.input[key], 0, wx.ALL | wx.EXPAND, 0)

        if self.group == 2:
            note = wx.StaticText(
                self,
                wx.ID_ANY,
                "Optionally define an SQL connection for Group 2 queries. "
                "This connection is read-only. "
                "Your main SQL connection (i.e., Group 1) always applies to "
                "Database Administrator and Importing.",
            )
            sizer_frame.Add(note, 0, wx.ALL, 10)
            sizer_frame.Add(self.sync_groups, 0, wx.LEFT, 10)

        sizer_frame.Add(grid_sizer, 0, wx.ALL, 10)
        sizer_frame.Add(self.db_type_radiobox, 0, wx.EXPAND | wx.ALL, 5)
        sizer_buttons.Add(self.button["ok"], 1, wx.ALL | wx.EXPAND, 5)
        sizer_buttons.Add(self.button["cancel"], 1, wx.ALL | wx.EXPAND, 5)
        sizer_echo.Add(
            self.button["echo"], 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 5
        )
        sizer_write_test.Add(
            self.button["write_test"], 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 5
        )
        sizer_frame.Add(sizer_echo, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 5)
        sizer_frame.Add(sizer_write_test, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 5)
        sizer_reload.Add(
            self.button["reload"], 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 5
        )
        sizer_frame.Add(sizer_reload, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 5)
        sizer_frame.Add(sizer_buttons, 0, wx.ALL | wx.EXPAND, 5)

        self.SetSizer(sizer_frame)
        self.Fit()
        if self.group == 2:
            note.Wrap(300)
            self.Fit()
        self.Layout()

    def load_sql_settings(self):
        self.set_host_items()

        if self.has_last_cnx:
            config = self.options.SQL_LAST_CNX_GRPS[self.group][
                self.selected_db_type
            ]
        else:
            config = self.options.DEFAULT_CNF[self.selected_db_type]

        self.clear_input()

        if self.selected_db_type == "sqlite":
            self.input["host"].SetValue(config["host"])
        else:
            for input_type in self.keys:
                if input_type in config:
                    self.input[input_type].SetValue(config[input_type])

    def button_echo(self, evt):
        if self.valid_sql_settings:
            wx.MessageBox(
                "Success!", "Echo SQL Database", wx.OK | wx.ICON_INFORMATION
            )
        else:
            wx.MessageBox(
                "Invalid credentials!",
                "Echo SQL Database",
                wx.OK | wx.ICON_WARNING,
            )

    def button_write_test(self, evt):
        results = self.write_test_results
        answers = {True: "Passed", False: "Failed", None: "N/A"}
        msg = "Write: %s\nDelete: %s" % (
            answers[results["write"]],
            answers[results["delete"]],
        )
        wx.MessageBox(msg, "SQL DB Write Test", wx.OK | wx.ICON_WARNING)

    def button_reload(self, evt):
        self.load_sql_settings()

    def write_successful_cnf(self):
        new_config = {
            key: self.input[key].GetValue()
            for key in self.keys
            if self.input[key].GetValue()
        }
        self.options.SQL_LAST_CNX_GRPS[self.group][
            self.selected_db_type
        ] = new_config
        self.options.DB_TYPE_GRPS[self.group] = self.selected_db_type

        if self.selected_db_type == "pgsql":
            new_host = self.input["host"].GetValue()
            if new_host in self.ip_history:
                self.ip_history.pop(self.ip_history.index(new_host))
            self.ip_history.insert(0, new_host)

        if self.group == 2:
            self.options.SYNC_SQL_CNX = bool(self.sync_groups.GetValue())

        self.options.save()

    @property
    def config(self):
        config = {
            key: self.input[key].GetValue()
            for key in self.keys
            if self.input[key].GetValue()
        }
        if self.selected_db_type == "pgsql":
            config["dbname"] = self.input["dbname"].GetValue()
        return config

    @property
    def valid_sql_settings(self):
        return echo_sql_db(self.config, db_type=self.selected_db_type)

    @property
    def write_test_results(self):
        return write_test(
            self.config, db_type=self.selected_db_type, group=self.group
        )

    def run(self):
        res = self.ShowModal()
        if res == wx.ID_OK:
            new_config = {
                key: self.input[key].GetValue()
                for key in self.keys
                if self.input[key].GetValue()
            }

            if echo_sql_db(new_config, db_type=self.selected_db_type):
                self.write_successful_cnf()
                with DVH_SQL(group=self.group) as cnx:
                    cnx.initialize_database()
            else:
                dlg = wx.MessageDialog(
                    self,
                    "Connection to database could not be established.",
                    "ERROR!",
                    wx.OK | wx.ICON_ERROR,
                )
                dlg.ShowModal()
        self.Destroy()

    def on_db_radio(self, *evt):
        self.load_sql_settings()
        self.button["reload"].Enable(self.has_last_cnx)

    @property
    def selected_db_type(self):
        return ["sqlite", "pgsql"][self.db_type_radiobox.GetSelection()]

    @property
    def unselected_db_type(self):
        return ["pgsql", "sqlite"][self.db_type_radiobox.GetSelection()]

    @property
    def has_last_cnx(self):
        return "host" in list(
            self.options.SQL_LAST_CNX_GRPS[self.group][self.selected_db_type]
        )

    def set_selected_db_type(self, db_type):
        self.db_type_radiobox.SetSelection({"sqlite": 0, "pgsql": 1}[db_type])

    def clear_input(self):
        for input_type in self.keys:
            self.input[input_type].SetValue("")
            if input_type != "host":
                self.input[input_type].Enable(self.selected_db_type == "pgsql")
                self.label[input_type].Enable(self.selected_db_type == "pgsql")

    def set_all_enable(self, status):
        for obj in self.input.values():
            obj.Enable(status)
        for obj in self.label.values():
            obj.Enable(status)

        for key in ["echo", "write_test", "reload"]:
            self.button[key].Enable(status)

        self.db_type_radiobox.Enable(status)

    def on_sync_groups(self, *evt):
        self.set_all_enable(not bool(self.sync_groups.GetValue()))
        self.load_sql_settings()


# ------------------------------------------------------
# Yes/No Dialogs based on tools.utilities.MessageDialog
#
# MessageDialog handles the layout and display, simply
# provide the wx parent, a message, and define over-ride
# action_yes and/or action_no for Yes/No button clicks
# ------------------------------------------------------
class DeleteAllData(MessageDialog):
    def __init__(self, parent, options):
        self.options = options
        MessageDialog.__init__(self, parent, "Delete All Data in Database")

    def action_yes(self):
        with DVH_SQL() as cnx:
            cnx.reinitialize_database()
        MoveFilesToInbox(
            self.parent, self.options.INBOX_DIR, self.options.IMPORTED_DIR
        )


class MoveFilesToInbox(MessageDialog):
    def __init__(self, parent, inbox_dir, imported_dir):
        self.parent = parent
        self.inbox_dir = inbox_dir
        self.imported_dir = imported_dir
        MessageDialog.__init__(self, parent, "Move files to inbox?")

    def action_yes(self):
        new_dir = join(
            self.inbox_dir,
            "previously_imported %s"
            % str(datetime.now())
            .split(".")[0]
            .replace(":", "-")
            .replace(" ", "_"),
        )
        rename(self.imported_dir, new_dir)
        mkdir(self.imported_dir)

    def action_no(self):
        DeleteImportedDirectory(self.parent, self.imported_dir)


class DeleteImportedDirectory(MessageDialog):
    def __init__(self, parent, directory):
        self.directory = directory
        MessageDialog.__init__(self, parent, "Delete Imported Directory?")

    def action_yes(self):
        delete_directory_contents(self.directory)


class MoveFiles(MessageDialog):
    def __init__(self, parent, files):
        self.parent = parent
        self.files = files
        MessageDialog.__init__(self, parent, "Move files to another location?")

    def action_yes(self):
        dlg = wx.DirDialog(
            self,
            "Choose directory",
            "",
            wx.DD_DEFAULT_STYLE | wx.DD_DIR_MUST_EXIST,
        )
        if dlg.ShowModal() == wx.ID_OK:
            move_files_to_new_path(self.files, dlg.GetPath())
        dlg.Destroy()

    def action_no(self):
        DeleteFiles(self.parent, self.files)


class DeleteFiles(MessageDialog):
    def __init__(self, parent, files):
        self.files = files
        MessageDialog.__init__(self, parent, "Delete associated files?")

    def action_yes(self):
        for f in self.files:
            delete_file(f)


class DeleteFilesFromQuery(MessageDialog):
    def __init__(self, parent, dicom_file_query):
        self.dicom_file_query = dicom_file_query
        MessageDialog.__init__(self, parent, "Delete associated files?")

    def action_yes(self):
        delete_imported_dicom_files(self.dicom_file_query)

    def action_no(self):
        MoveFilesFromQuery(self.parent, self.dicom_file_query)


class MoveFilesFromQuery(MessageDialog):
    def __init__(self, parent, dicom_file_query):
        self.dicom_file_query = dicom_file_query
        MessageDialog.__init__(self, parent, "Move associated files?")

    def action_yes(self):
        dlg = wx.DirDialog(
            self.parent,
            "Choose directory",
            "",
            wx.DD_DEFAULT_STYLE | wx.DD_DIR_MUST_EXIST,
        )
        if dlg.ShowModal() == wx.ID_OK:
            move_imported_dicom_files(self.dicom_file_query, dlg.GetPath())
        dlg.Destroy()


class RebuildDB(MessageDialog):
    def __init__(self, parent, roi_map, options):
        self.roi_map = roi_map
        self.options = options
        MessageDialog.__init__(self, parent, "Rebuild Database from DICOM")

    def action_yes(self):
        with DVH_SQL() as cnx:
            cnx.reinitialize_database()

        ImportDicomFrame(
            self.roi_map,
            self.options,
            inbox=self.options.IMPORTED_DIR,
            auto_parse=True,
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# dvha_app.py
"""
Script to start DVH Analytics
"""
# Copyright (c) 2016-2020 Dan Cutright
# This file is part of DVH Analytics, released under a BSD license.
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics


import dvha.main
from dvha.paths import set_phantom_js_path_environment
import multiprocessing

if __name__ == "__main__":

    # Required if running from PyInstaller freeze
    # Multiprocessing library used for dose summation to avoid memory
    # allocation issues
    multiprocessing.freeze_support()

    # SVG export with Bokeh requires PhantomJS
    # Edit PATH environment so Bokeh can find phantomjs binary
    set_phantom_js_path_environment()

    # Begin the main application
    dvha.main.start()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# dialogs.dvha_app.py
"""
Dialogs used in the main view of DVHA (e.g., query design, importing, etc.)
"""
# Copyright (c) 2016-2019 Dan Cutright
# This file is part of DVH Analytics, released under a BSD license.
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics

import wx
import wx.adv
from dateutil.parser import parse as parse_date
import matplotlib.colors as plot_colors
import numpy as np
import time
from os.path import isdir
from dvha.tools.utilities import (
    get_selected_listctrl_items,
    MessageDialog,
    get_window_size,
    get_installed_python_libraries,
    set_msw_background_color,
    set_frame_icon,
    backup_sqlite_db,
)
from dvha.db import sql_columns
from dvha.db.sql_connector import DVH_SQL
from dvha.models.data_table import DataTable
from dvha.paths import LICENSE_PATH
from dvha.options import DefaultOptions
from dvha.tools.errors import ErrorDialog


class DatePicker(wx.Dialog):
    """
    Pop-up window to select a date to ensure proper formatting (over typing in a date into a text_ctrl directly)
    """

    def __init__(
        self, title="", initial_date=None, action=None, sql_date_format=False
    ):
        """
        :param title: optional title for the wx.Dialog
        :type title: str
        :param initial_date: optional initial date
        :type initial_date: str
        :param action: pointer to function to be executed on wx.ID_OK or
        :param sql_date_format: Set to True for SQL compatible format
        :type sql_date_format: bool
        """
        wx.Dialog.__init__(self, None, title=title)

        self.sql_date_format = sql_date_format

        self.calendar_ctrl = wx.adv.CalendarCtrl(
            self,
            wx.ID_ANY,
            style=wx.adv.CAL_SHOW_HOLIDAYS | wx.adv.CAL_SHOW_SURROUNDING_WEEKS,
        )
        if initial_date and initial_date.lower() != "none":
            self.calendar_ctrl.SetDate(parse_date(initial_date))

        self.button = {
            "ok": wx.Button(self, wx.ID_OK, "OK"),
            "delete": wx.Button(self, wx.ID_ANY, "Delete"),
            "cancel": wx.Button(self, wx.ID_CANCEL, "Cancel"),
        }

        self.button["delete"].Enable(not sql_date_format)

        self.none = (
            False  # If True after close of this dialog, user deleted the value
        )

        self.action = action

        self.__do_layout()
        self.__do_bind()

        self.run()

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_main = wx.BoxSizer(wx.VERTICAL)
        sizer_buttons = wx.BoxSizer(wx.HORIZONTAL)
        sizer_main.Add(self.calendar_ctrl, 0, wx.ALIGN_CENTER | wx.ALL, 5)
        for button in self.button.values():
            sizer_buttons.Add(button, 0, wx.ALL, 5)
        sizer_main.Add(
            sizer_buttons, 1, wx.ALIGN_CENTER | wx.BOTTOM | wx.TOP, 10
        )
        sizer_wrapper.Add(sizer_main, 1, wx.ALL | wx.EXPAND, 10)
        self.SetSizer(sizer_wrapper)
        sizer_wrapper.Fit(self)
        self.Layout()
        self.Center()

    def __do_bind(self):
        self.Bind(
            wx.EVT_BUTTON, self.on_delete, id=self.button["delete"].GetId()
        )

    @property
    def date(self):
        if self.none:
            return ""
        date = self.calendar_ctrl.GetDate()
        date = "%s/%s/%s" % (date.month + 1, date.day, date.year)
        if self.sql_date_format:
            return str(parse_date(date).date())
        return date

    def on_delete(self, evt):
        self.none = True
        self.Close()  # resolve dialog with wx.ID_CANCEL

    def run(self):
        res = self.ShowModal()
        if res == wx.ID_OK or (res == wx.ID_CANCEL and self.none):
            if self.action is not None:
                self.action(self.date)
        self.Destroy()


class AddEndpointDialog(wx.Dialog):
    """
    Add a new column to the endpoint table of the Endpoint tab
    """

    def __init__(self):
        wx.Dialog.__init__(self, None, title="Add Endpoint")

        self.combo_box_output = wx.ComboBox(
            self,
            wx.ID_ANY,
            choices=["Dose (Gy)", "Dose(%)", "Volume (cc)", "Volume (%)"],
            style=wx.CB_DROPDOWN | wx.CB_READONLY,
        )
        self.text_input = wx.TextCtrl(self, wx.ID_ANY, "")
        self.radio_box_units = wx.RadioBox(
            self,
            wx.ID_ANY,
            "",
            choices=["cc ", "% "],
            majorDimension=1,
            style=wx.RA_SPECIFY_ROWS,
        )
        self.button_ok = wx.Button(self, wx.ID_OK, "OK")
        self.button_ok.Disable()
        self.button_cancel = wx.Button(self, wx.ID_CANCEL, "Cancel")

        self.__do_bind()
        self.__set_properties()
        self.__do_layout()

    def __set_properties(self):
        self.radio_box_units.SetSelection(0)
        self.combo_box_output.SetValue("Dose (Gy)")

    def __do_bind(self):
        self.Bind(
            wx.EVT_COMBOBOX,
            self.combo_box_ticker,
            id=self.combo_box_output.GetId(),
        )
        self.Bind(
            wx.EVT_TEXT, self.text_input_ticker, id=self.text_input.GetId()
        )
        self.Bind(
            wx.EVT_RADIOBOX,
            self.radio_box_ticker,
            id=self.radio_box_units.GetId(),
        )

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_buttons_wrapper = wx.BoxSizer(wx.HORIZONTAL)
        sizer_buttons = wx.BoxSizer(wx.HORIZONTAL)
        sizer_input = wx.StaticBoxSizer(
            wx.StaticBox(self, wx.ID_ANY, ""), wx.HORIZONTAL
        )
        sizer_input_units = wx.BoxSizer(wx.VERTICAL)
        sizer_input_value = wx.BoxSizer(wx.VERTICAL)
        sizer_output = wx.BoxSizer(wx.VERTICAL)

        label_ouput = wx.StaticText(self, wx.ID_ANY, "Output:")
        sizer_output.Add(label_ouput, 0, wx.BOTTOM | wx.EXPAND, 8)
        sizer_output.Add(self.combo_box_output, 0, wx.EXPAND, 0)
        sizer_input.Add(sizer_output, 1, wx.ALL | wx.EXPAND, 5)

        self.label_input_value = wx.StaticText(
            self, wx.ID_ANY, "Input Volume (cc):"
        )
        sizer_input_value.Add(
            self.label_input_value, 0, wx.BOTTOM | wx.EXPAND, 8
        )
        sizer_input_value.Add(self.text_input, 0, wx.EXPAND | wx.LEFT, 5)
        sizer_input.Add(sizer_input_value, 1, wx.ALL | wx.EXPAND, 5)

        label_input_units = wx.StaticText(self, wx.ID_ANY, "Input Units:")
        sizer_input_units.Add(label_input_units, 0, wx.BOTTOM | wx.EXPAND, 3)
        sizer_input_units.Add(self.radio_box_units, 0, wx.EXPAND, 0)
        sizer_input.Add(sizer_input_units, 1, wx.ALL | wx.EXPAND, 5)
        sizer_wrapper.Add(sizer_input, 0, wx.ALL | wx.EXPAND, 10)

        self.text_short_hand = wx.StaticText(self, wx.ID_ANY, "Short-hand: ")
        sizer_wrapper.Add(self.text_short_hand, 0, wx.ALL, 5)
        sizer_buttons.Add(self.button_ok, 0, wx.ALL, 5)
        sizer_buttons.Add(self.button_cancel, 0, wx.ALL | wx.EXPAND, 5)
        sizer_buttons_wrapper.Add(sizer_buttons, 0, wx.ALL | wx.EXPAND, 5)
        sizer_wrapper.Add(
            sizer_buttons_wrapper, 0, wx.ALIGN_CENTER | wx.ALL, 5
        )

        self.SetSizer(sizer_wrapper)
        sizer_wrapper.Fit(self)
        self.Layout()
        self.Center()

    def combo_box_ticker(self, evt):
        self.update_radio_box_choices()
        self.update_label_input()
        self.update_short_hand()

    def text_input_ticker(self, evt):
        self.update_short_hand()

    def radio_box_ticker(self, evt):
        self.update_label_input()
        self.update_short_hand()

    def update_label_input(self):
        new_label = "%s (%s):" % (
            ["Input Dose", "Input Volume"][
                "Dose" in self.combo_box_output.GetValue()
            ],
            self.radio_box_units.GetItemLabel(
                self.radio_box_units.GetSelection()
            ).strip(),
        )
        self.label_input_value.SetLabelText(new_label)

    def update_radio_box_choices(self):
        choice_1 = ["Gy", "cc"]["Dose" in self.combo_box_output.GetValue()]
        self.radio_box_units.SetItemLabel(0, choice_1)

    def update_short_hand(self):
        short_hand = "Short-hand: "
        value = self.text_input.GetValue()
        if value:
            prepend = ["V_", "D_"]["Dose" in self.combo_box_output.GetValue()]
            units = self.radio_box_units.GetItemLabel(
                self.radio_box_units.GetSelection()
            ).strip()
            short_hand = short_hand + prepend + value + units
        self.text_short_hand.SetLabelText(short_hand)
        self.set_button_ok_enable(value)

    def set_button_ok_enable(self, value):
        try:
            if value.isdigit():
                value = int(value)
            else:
                value = float(value)
        except ValueError:
            self.text_short_hand.SetLabelText("Short-hand: ")
        self.button_ok.Enable(type(value) in [int, float])

    @property
    def is_endpoint_valid(self):
        return bool(len(self.short_hand_label))

    @property
    def short_hand_label(self):
        return (
            self.text_short_hand.GetLabel().replace("Short-hand: ", "").strip()
        )

    @property
    def output_type(self):
        return ["absolute", "relative"][
            "%" in self.combo_box_output.GetValue()
        ]

    @property
    def input_type(self):
        return ["absolute", "relative"][self.radio_box_units.GetSelection()]

    @property
    def units_in(self):
        return (
            self.radio_box_units.GetItemLabel(
                self.radio_box_units.GetSelection()
            )
            .replace("%", "")
            .strip()
        )

    @property
    def units_out(self):
        return (
            self.combo_box_output.GetValue()
            .split("(")[1][:-1]
            .replace("%", "")
            .strip()
        )

    @property
    def input_value(self):
        try:
            return float(self.text_input.GetValue())
        except ValueError:
            return 0.0

    @property
    def endpoint_row(self):
        return [
            self.short_hand_label,
            self.output_type,
            self.input_type,
            self.input_value,
            self.units_in,
            self.units_out,
        ]


class SelectFromListDialog(wx.Dialog):
    """
    Select an item in a list
    """

    def __init__(
        self,
        title,
        column,
        choices,
        exclude=None,
        size=None,
        column_width=200,
        selections=None,
        single_mode=False,
    ):
        wx.Dialog.__init__(self, None, title=title)

        self.column = column
        self.choices = choices
        self.exclude = [exclude, []][exclude is None]
        self.size = size
        self.column_width = column_width
        self.single_mode = single_mode

        style = (
            wx.LC_SINGLE_SEL | wx.LC_REPORT if single_mode else wx.LC_REPORT
        )
        self.list_ctrl = wx.ListCtrl(self, wx.ID_ANY, style=style)
        if not self.single_mode:
            self.button_select_all = wx.Button(self, wx.ID_ANY, "Select All")
            self.button_deselect_all = wx.Button(
                self, wx.ID_ANY, "Deselect All"
            )
        self.button_ok = wx.Button(self, wx.ID_OK, "OK")
        self.button_cancel = wx.Button(self, wx.ID_CANCEL, "Cancel")

        if not self.single_mode:
            self.Bind(
                wx.EVT_BUTTON,
                self.select_all,
                id=self.button_select_all.GetId(),
            )
            self.Bind(
                wx.EVT_BUTTON,
                self.deselect_all,
                id=self.button_deselect_all.GetId(),
            )

        self.__set_properties()
        self.__do_layout()

        if selections:
            self.set_selection(selections)

    def __set_properties(self):
        self.list_ctrl.AppendColumn(
            self.column, format=wx.LIST_FORMAT_LEFT, width=self.column_width
        )

        for choice in self.choices:
            if choice not in self.exclude:
                self.list_ctrl.InsertItem(50000, choice)

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_ok_cancel = wx.BoxSizer(wx.HORIZONTAL)
        sizer_select = wx.StaticBoxSizer(
            wx.StaticBox(self, wx.ID_ANY, ""), wx.VERTICAL
        )
        sizer_select_buttons = wx.BoxSizer(wx.HORIZONTAL)

        sizer_select.Add(self.list_ctrl, 1, wx.ALL | wx.EXPAND, 5)
        if not self.single_mode:
            sizer_select_buttons.Add(self.button_select_all, 0, wx.ALL, 5)
            sizer_select_buttons.Add(self.button_deselect_all, 0, wx.ALL, 5)
        sizer_select.Add(sizer_select_buttons, 0, wx.ALIGN_CENTER | wx.ALL, 0)
        sizer_wrapper.Add(sizer_select, 1, wx.ALL | wx.EXPAND, 5)

        sizer_ok_cancel.Add(self.button_ok, 0, wx.ALL, 5)
        sizer_ok_cancel.Add(self.button_cancel, 0, wx.ALL, 5)
        sizer_wrapper.Add(sizer_ok_cancel, 0, wx.ALIGN_CENTER | wx.ALL, 5)

        self.SetSizer(sizer_wrapper)
        if self.size:
            self.SetSize(self.size)
        else:
            sizer_wrapper.Fit(self)
        self.Layout()
        self.Center()

    @property
    def selected_indices(self):
        return get_selected_listctrl_items(self.list_ctrl)

    @property
    def selected_values(self):
        return [
            self.list_ctrl.GetItem(i, 0).GetText()
            for i in self.selected_indices
        ]

    @property
    def item_count(self):
        return len(self.choices) - len(self.exclude)

    def select_all(self, evt):
        self.apply_global_selection()

    def deselect_all(self, evt):
        self.apply_global_selection(on=0)

    def apply_global_selection(self, on=1):
        for i in range(self.item_count):
            self.list_ctrl.Select(i, on=on)

    def set_selection(self, selections):
        for selection in selections:
            if selection in self.choices:
                index = self.choices.index(selection)
                self.list_ctrl.Select(index, on=True)


class DelEndpointDialog(SelectFromListDialog):
    def __init__(self, endpoints):
        SelectFromListDialog.__init__(
            self,
            "Delete Endpoint",
            "Endpoints",
            endpoints,
            exclude={"MRN", "Tx Site", "ROI Name", "Volume (cc)"},
        )


class SelectRegressionVariablesDialog(SelectFromListDialog):
    def __init__(
        self, dependent_variable, independent_variable_choices, selections=None
    ):
        self.dependent_variable = dependent_variable
        self.independent_variable_choices = independent_variable_choices
        size = get_window_size(1, 0.8)
        SelectFromListDialog.__init__(
            self,
            "Select Variables for %s" % dependent_variable,
            "Independent Variables",
            independent_variable_choices,
            size=(350, size[1]),
            column_width=300,
            selections=selections,
        )


class SelectMLVarDialog(SelectFromListDialog):
    def __init__(
        self,
        variable_choices,
        selections=None,
        single_mode=True,
        algorithm=False,
        fit=False,
    ):
        if algorithm:
            column = "Algorithm"
            self.variable_choices = sorted(list(variable_choices))
        else:
            column = "Select %sependent Variable%s" % (
                ["D", "Ind"][single_mode],
                ["s", ""][single_mode],
            )
            self.variable_choices = variable_choices
        size = (350, get_window_size(1, 0.5)[1]) if not fit else None
        SelectFromListDialog.__init__(
            self,
            "Machine Learning",
            column,
            variable_choices,
            size=size,
            column_width=300,
            selections=selections,
            single_mode=single_mode,
        )


def query_dlg(parent, query_type, set_values=False):
    """
    Function to create either QueryCategoryDialog or QueryNumericalDialog
    :param parent: pointer to app object
    :type parent: DVHAMainFrame
    :param query_type: either 'categorical' or 'numerical'
    :type query_type: str
    :param set_values: If True, pre-populate dialog values with the currently selected row (i.e., edit a row)
    :type set_values: bool
    """
    dlg = {
        "categorical": QueryCategoryDialog,
        "numerical": QueryNumericalDialog,
    }[query_type]()
    data_table = {
        "categorical": parent.data_table_categorical,
        "numerical": parent.data_table_numerical,
    }[query_type]
    selected_index = {
        "categorical": parent.selected_index_categorical,
        "numerical": parent.selected_index_numerical,
    }[query_type]
    if set_values:
        dlg.set_values(data_table.get_row(selected_index))

    res = dlg.ShowModal()
    if res == wx.ID_OK:
        row = dlg.get_values()
        if set_values:
            data_table.edit_row(row, selected_index)
        else:
            data_table.append_row(row)
        parent.update_all_query_buttons()
    dlg.Destroy()


class QueryCategoryDialog(wx.Dialog):
    """
    Add/Edit query parameters for categorical data
    """

    def __init__(self):
        wx.Dialog.__init__(self, None, title="Query by Categorical Data")

        self.selector_categories = sql_columns.categorical

        selector_options = list(self.selector_categories)
        selector_options.sort()

        self.combo_box_1 = wx.ComboBox(
            self,
            wx.ID_ANY,
            choices=selector_options,
            style=wx.CB_DROPDOWN | wx.CB_READONLY,
        )
        self.combo_box_2 = wx.ComboBox(
            self, wx.ID_ANY, choices=[], style=wx.CB_DROPDOWN | wx.CB_READONLY
        )
        self.checkbox_1 = wx.CheckBox(self, wx.ID_ANY, "Exclude")
        self.button_OK = wx.Button(self, wx.ID_OK, "OK")
        self.button_cancel = wx.Button(self, wx.ID_CANCEL, "Cancel")

        self.__do_layout()

        self.combo_box_1.SetValue("ROI Institutional Category")
        self.update_category_2(None)
        self.Bind(
            wx.EVT_COMBOBOX,
            self.update_category_2,
            id=self.combo_box_1.GetId(),
        )

        self.Fit()
        self.Center()

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_vbox = wx.BoxSizer(wx.VERTICAL)
        sizer_ok_cancel = wx.BoxSizer(wx.HORIZONTAL)
        sizer_widgets = wx.StaticBoxSizer(
            wx.StaticBox(self, wx.ID_ANY, ""), wx.HORIZONTAL
        )
        sizer_category_2 = wx.BoxSizer(wx.VERTICAL)
        sizer_category_1 = wx.BoxSizer(wx.VERTICAL)
        label_category_1 = wx.StaticText(self, wx.ID_ANY, "Category 1:")
        sizer_category_1.Add(label_category_1, 0, wx.ALL | wx.EXPAND, 5)
        sizer_category_1.Add(self.combo_box_1, 0, wx.ALL, 5)
        sizer_widgets.Add(sizer_category_1, 1, wx.EXPAND, 0)
        label_category_2 = wx.StaticText(self, wx.ID_ANY, "Category 2:")
        sizer_category_2.Add(label_category_2, 0, wx.ALL | wx.EXPAND, 5)
        sizer_category_2.Add(self.combo_box_2, 0, wx.EXPAND | wx.ALL, 5)
        sizer_widgets.Add(sizer_category_2, 1, wx.EXPAND, 0)
        sizer_widgets.Add(self.checkbox_1, 0, wx.ALL | wx.EXPAND, 5)
        sizer_vbox.Add(sizer_widgets, 0, wx.ALL | wx.EXPAND, 5)
        sizer_ok_cancel.Add(
            self.button_OK, 0, wx.EXPAND | wx.LEFT | wx.RIGHT, 5
        )
        sizer_ok_cancel.Add(self.button_cancel, 0, wx.LEFT | wx.RIGHT, 5)
        sizer_vbox.Add(sizer_ok_cancel, 0, wx.ALIGN_RIGHT | wx.ALL, 5)
        sizer_wrapper.Add(sizer_vbox, 0, wx.ALL | wx.EXPAND, 5)
        self.SetSizer(sizer_wrapper)

    def update_category_2(self, evt):
        key = self.combo_box_1.GetValue()
        table = self.selector_categories[key]["table"]
        col = self.selector_categories[key]["var_name"]
        with DVH_SQL() as cnx:
            options = cnx.get_unique_values(table, col)
        self.combo_box_2.Clear()
        self.combo_box_2.Append(options)
        if options:
            self.combo_box_2.SetValue(options[0])

    def set_category_1(self, value):
        self.combo_box_1.SetValue(value)
        self.update_category_2(None)

    def set_category_2(self, value):
        self.combo_box_2.SetValue(value)

    def set_check_box_not(self, value):
        self.checkbox_1.SetValue(value)

    def set_values(self, values):
        self.set_category_1(values[0])
        self.set_category_2(values[1])
        self.set_check_box_not({"Include": False, "Exclude": True}[values[2]])

    def get_values(self):
        return [
            self.combo_box_1.GetValue(),
            self.combo_box_2.GetValue(),
            ["Include", "Exclude"][self.checkbox_1.GetValue()],
        ]


class QueryNumericalDialog(wx.Dialog):
    """
    Add/Edit query parameters for numerical data
    """

    def __init__(self):
        wx.Dialog.__init__(self, None, title="Query by Numerical Data")

        self.numerical_categories = sql_columns.numerical

        numerical_options = list(self.numerical_categories)
        numerical_options.sort()

        self.combo_box_1 = wx.ComboBox(
            self,
            wx.ID_ANY,
            choices=numerical_options,
            style=wx.CB_DROPDOWN | wx.CB_READONLY,
        )
        self.text_ctrl_min = wx.TextCtrl(self, wx.ID_ANY, "")
        self.text_ctrl_max = wx.TextCtrl(self, wx.ID_ANY, "")
        self.checkbox_1 = wx.CheckBox(self, wx.ID_ANY, "Exclude")
        self.button_OK = wx.Button(self, wx.ID_OK, "OK")
        self.button_cancel = wx.Button(self, wx.ID_CANCEL, "Cancel")
        self.button_date_picker = wx.Button(self, wx.ID_ANY, "Edit Dates")

        self.__do_layout()
        self.__do_bind()

        self.combo_box_1.SetValue("ROI Max Dose")
        self.update_range(None)

    def __do_bind(self):
        self.Bind(
            wx.EVT_COMBOBOX, self.update_range, id=self.combo_box_1.GetId()
        )
        self.Bind(
            wx.EVT_BUTTON,
            self.on_date_picker,
            id=self.button_date_picker.GetId(),
        )

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_vbox = wx.BoxSizer(wx.VERTICAL)
        sizer_ok_cancel = wx.BoxSizer(wx.HORIZONTAL)
        sizer_widgets = wx.StaticBoxSizer(
            wx.StaticBox(self, wx.ID_ANY, ""), wx.HORIZONTAL
        )
        sizer_max = wx.BoxSizer(wx.VERTICAL)
        sizer_min = wx.BoxSizer(wx.VERTICAL)
        sizer_category_1 = wx.BoxSizer(wx.VERTICAL)
        label_category = wx.StaticText(self, wx.ID_ANY, "Category:")
        sizer_category_1.Add(label_category, 0, wx.ALL | wx.EXPAND, 5)
        sizer_category_1.Add(self.combo_box_1, 0, wx.ALL, 5)
        sizer_widgets.Add(sizer_category_1, 1, wx.EXPAND, 0)
        self.label_min = wx.StaticText(self, wx.ID_ANY, "Min:")
        sizer_min.Add(self.label_min, 0, wx.ALL | wx.EXPAND, 5)
        sizer_min.Add(self.text_ctrl_min, 0, wx.ALL, 5)
        sizer_widgets.Add(sizer_min, 0, wx.EXPAND, 0)
        self.label_max = wx.StaticText(self, wx.ID_ANY, "Max:")
        sizer_max.Add(self.label_max, 0, wx.ALL | wx.EXPAND, 5)
        sizer_max.Add(self.text_ctrl_max, 0, wx.ALL, 5)
        sizer_widgets.Add(sizer_max, 0, wx.EXPAND, 0)
        sizer_widgets.Add(self.checkbox_1, 0, wx.ALL | wx.EXPAND, 5)
        sizer_vbox.Add(sizer_widgets, 0, wx.ALL | wx.EXPAND, 5)
        sizer_ok_cancel.Add(
            self.button_date_picker, 0, wx.EXPAND | wx.LEFT | wx.RIGHT, 5
        )
        sizer_ok_cancel.Add(
            self.button_OK, 0, wx.EXPAND | wx.LEFT | wx.RIGHT, 5
        )
        sizer_ok_cancel.Add(self.button_cancel, 0, wx.LEFT | wx.RIGHT, 5)
        sizer_vbox.Add(sizer_ok_cancel, 0, wx.ALIGN_RIGHT | wx.ALL, 5)
        sizer_wrapper.Add(sizer_vbox, 0, wx.ALL | wx.EXPAND, 5)
        self.SetSizer(sizer_wrapper)

        self.Fit()
        self.Center()

    def update_range(self, evt):
        key = self.combo_box_1.GetValue()
        table = self.numerical_categories[key]["table"]
        col = self.numerical_categories[key]["var_name"]
        units = self.numerical_categories[key]["units"]
        with DVH_SQL() as cnx:
            min_value = cnx.get_min_value(table, col)
            max_value = cnx.get_max_value(table, col)

        self.button_date_picker.Enable("date" in key.lower())
        self.text_ctrl_min.Enable("date" not in key.lower())
        self.text_ctrl_max.Enable("date" not in key.lower())

        self.set_min_value(min_value)
        self.set_max_value(max_value)

        if "date" in key.lower():
            self.update_min_max_text("Start:", "End:")
            self.on_date_picker()
        elif units:
            self.update_min_max_text("Min (%s):" % units, "Max (%s):" % units)
        else:
            self.update_min_max_text("Min:", "Max:")

        self.clean_numeric_input()

    def update_min_max_text(self, min_text, max_text):
        self.label_min.SetLabelText(min_text)
        self.label_max.SetLabelText(max_text)

    def set_category(self, value):
        self.combo_box_1.SetValue(value)
        self.update_range(None)

    def set_min_value(self, value):
        self.text_ctrl_min.SetValue(str(value))

    def set_max_value(self, value):
        self.text_ctrl_max.SetValue(str(value))

    def set_check_box_not(self, value):
        self.checkbox_1.SetValue(value)

    def set_values(self, values):
        self.set_category(values[0])
        self.set_min_value(str(values[1]))
        self.set_max_value(str(values[2]))
        self.set_check_box_not({"Include": False, "Exclude": True}[values[3]])

    def get_values(self):
        return [
            self.combo_box_1.GetValue(),
            self.text_ctrl_min.GetValue(),
            self.text_ctrl_max.GetValue(),
            ["Include", "Exclude"][self.checkbox_1.GetValue()],
        ]

    def validated_text(self, input_type):
        old_value = {
            "min": self.text_ctrl_min.GetValue(),
            "max": self.text_ctrl_max.GetValue(),
        }[input_type]

        try:
            new_value = float(old_value)
        except ValueError:
            key = self.combo_box_1.GetValue()
            table = self.numerical_categories[key]["table"]
            col = self.numerical_categories[key]["var_name"]
            with DVH_SQL() as cnx:
                if input_type == "min":
                    new_value = cnx.get_min_value(table, col)
                else:
                    new_value = cnx.get_max_value(table, col)
        return new_value

    def on_date_picker(self, *evt):
        DatePicker(
            initial_date=self.text_ctrl_min.GetValue(),
            title="Start Date",
            action=self.set_min_value,
            sql_date_format=True,
        )
        time.sleep(
            0.3
        )  # Immediately loading the next DatePicker looks like a glitch
        DatePicker(
            initial_date=self.text_ctrl_max.GetValue(),
            title="End Date",
            action=self.set_max_value,
            sql_date_format=True,
        )

    def clean_numeric_input(self):
        for text_ctrl in [self.text_ctrl_min, self.text_ctrl_max]:
            try:
                value = float(text_ctrl.GetValue())
                func = [np.floor, np.ceil][text_ctrl == self.text_ctrl_max]
                text_ctrl.SetValue("%0.2f" % func(value))
            except ValueError:
                pass


class UserSettings(wx.Frame):
    """
    Customize directories and visual settings for DVHA
    """

    def __init__(self, parent):
        """
        :param parent: main application frame
        """
        wx.Frame.__init__(self, None, title="User Settings")

        self.parent = parent
        self.options = parent.options
        self.options.edit_detected = False

        colors = list(plot_colors.cnames)
        colors.sort()

        color_variables = self.get_option_choices("COLOR")
        size_variables = self.get_option_choices("SIZE")
        width_variables = self.get_option_choices("LINE_WIDTH")
        line_dash_variables = self.get_option_choices("LINE_DASH")
        alpha_variables = self.get_option_choices("ALPHA")

        line_style_options = [
            "solid",
            "dashed",
            "dotted",
            "dotdash",
            "dashdot",
        ]

        # self.SetSize((500, 580))
        self.text_ctrl_inbox = wx.TextCtrl(
            self, wx.ID_ANY, "", style=wx.TE_DONTWRAP
        )
        self.button_inbox = wx.Button(self, wx.ID_ANY, u"…")
        self.text_ctrl_imported = wx.TextCtrl(
            self, wx.ID_ANY, "", style=wx.TE_DONTWRAP
        )
        self.button_imported = wx.Button(self, wx.ID_ANY, u"…")

        self.checkbox_dicom_dvh = wx.CheckBox(
            self, wx.ID_ANY, "Import DICOM DVH if available"
        )
        self.dvh_bin_width_input = wx.TextCtrl(
            self, wx.ID_ANY, str(self.options.dvh_bin_width)
        )
        self.dvh_bin_max_dose = wx.TextCtrl(self, wx.ID_ANY, "")
        self.dvh_bin_max_dose_units = wx.ComboBox(
            self,
            wx.ID_ANY,
            choices=self.options.dvh_bin_max_dose_options,
            style=wx.CB_DROPDOWN | wx.CB_READONLY,
        )
        self.dvh_small_volume_threshold = wx.SpinCtrl(
            self, wx.ID_ANY, "10", min=1, max=50, style=wx.SP_ARROW_KEYS
        )
        self.dvh_segments_between = wx.SpinCtrl(
            self, wx.ID_ANY, "10", min=0, max=20, style=wx.SP_ARROW_KEYS
        )
        self.dvh_high_resolution = wx.ComboBox(
            self,
            wx.ID_ANY,
            choices=self.options.DVH_HIGH_RESOLUTION_FACTOR_OPTIONS,
            style=wx.CB_DROPDOWN | wx.CB_READONLY,
        )
        self.combo_box_colors_category = wx.ComboBox(
            self,
            wx.ID_ANY,
            choices=color_variables,
            style=wx.CB_DROPDOWN | wx.CB_READONLY,
        )
        self.combo_box_colors_selection = wx.ComboBox(
            self,
            wx.ID_ANY,
            choices=colors,
            style=wx.CB_DROPDOWN | wx.CB_READONLY,
        )
        self.combo_box_sizes_category = wx.ComboBox(
            self,
            wx.ID_ANY,
            choices=size_variables,
            style=wx.CB_DROPDOWN | wx.CB_READONLY,
        )
        self.spin_ctrl_sizes_input = wx.SpinCtrl(
            self, wx.ID_ANY, "0", min=0, max=20, style=wx.SP_ARROW_KEYS
        )
        self.combo_box_line_widths_category = wx.ComboBox(
            self,
            wx.ID_ANY,
            choices=width_variables,
            style=wx.CB_DROPDOWN | wx.CB_READONLY,
        )
        self.spin_ctrl_line_widths_input = wx.SpinCtrl(
            self, wx.ID_ANY, "0", min=0, max=10, style=wx.SP_ARROW_KEYS
        )
        self.combo_box_line_styles_category = wx.ComboBox(
            self,
            wx.ID_ANY,
            choices=line_dash_variables,
            style=wx.CB_DROPDOWN | wx.CB_READONLY,
        )
        self.combo_box_line_styles_selection = wx.ComboBox(
            self,
            wx.ID_ANY,
            choices=line_style_options,
            style=wx.CB_DROPDOWN | wx.CB_READONLY,
        )
        self.combo_box_alpha_category = wx.ComboBox(
            self,
            wx.ID_ANY,
            choices=alpha_variables,
            style=wx.CB_DROPDOWN | wx.CB_READONLY,
        )
        self.spin_ctrl_alpha_input = wx.SpinCtrlDouble(
            self, wx.ID_ANY, "0", min=0, max=1.0, style=wx.SP_ARROW_KEYS
        )
        self.button_restore_defaults = wx.Button(
            self, wx.ID_ANY, "Restore Defaults"
        )
        self.button_ok = wx.Button(self, wx.ID_OK, "OK")
        self.button_cancel = wx.Button(self, wx.ID_CANCEL, "Cancel")
        self.button_apply = wx.Button(self, wx.ID_ANY, "Apply")

        self.__set_properties()
        self.__do_layout()
        self.__do_bind()

        self.refresh_options()
        self.load_paths()

        self.is_edited = False

        set_msw_background_color(self)
        set_frame_icon(self)

    def __set_properties(self):
        self.text_ctrl_inbox.SetToolTip(
            "Default directory for batch processing of incoming DICOM files"
        )
        self.text_ctrl_inbox.SetMinSize((100, 21))
        self.button_inbox.SetMinSize((40, 21))
        self.text_ctrl_imported.SetToolTip(
            "Directory for post-processed DICOM files"
        )
        self.text_ctrl_imported.SetMinSize((100, 21))
        self.button_imported.SetMinSize((40, 21))
        self.checkbox_dicom_dvh.SetValue(self.options.USE_DICOM_DVH)
        self.checkbox_dicom_dvh.SetToolTip(
            "If a DICOM RT-Dose file has a DVH Sequence, use this DVH instead of "
            "recalculating during import."
        )
        self.dvh_bin_width_input.SetToolTip("Value must be an integer.")
        self.dvh_bin_width_input.SetMinSize((50, 21))
        self.dvh_bin_max_dose.SetToolTip(
            "Prevent memory issues if dose grid contains very large, unrealistic doses"
        )
        self.dvh_bin_max_dose.SetValue(
            str(
                self.options.dvh_bin_max_dose[
                    self.options.dvh_bin_max_dose_units
                ]
            )
        )
        self.dvh_bin_max_dose_units.SetValue(
            self.options.dvh_bin_max_dose_units
        )
        self.dvh_small_volume_threshold.SetToolTip(
            "If ROI volume is less than this value, it will be recalculated with a "
            "resolution of the dose grid spacing divided by 16"
        )
        self.dvh_small_volume_threshold.SetValue(
            str(self.options.DVH_SMALL_VOLUME_THRESHOLD)
        )
        self.dvh_segments_between.SetToolTip(
            "If ROI volume is less than threshold, it will be recalculated with a this many "
            "segments interpolated between slices"
        )
        self.dvh_segments_between.SetValue(
            str(self.options.DVH_HIGH_RESOLUTION_SEGMENTS_BETWEEN)
        )
        self.dvh_high_resolution.SetToolTip(
            "If ROI volume is less than the volume threshold, the in-plane resolution will be "
            "increased by this factor (e.g., interpolate in-between the dose grid)"
        )
        self.dvh_high_resolution.SetValue(
            str(self.options.DVH_HIGH_RESOLUTION_FACTOR)
        )
        # self.dvh_bin_width_input.SetMinSize((50, 22))
        # self.dvh_bin_max_dose_units.SetMinSize((50, 22))
        self.combo_box_colors_category.SetMinSize(
            (250, self.combo_box_colors_category.GetSize()[1])
        )
        self.combo_box_colors_selection.SetMinSize(
            (145, self.combo_box_colors_selection.GetSize()[1])
        )
        self.combo_box_sizes_category.SetMinSize(
            (250, self.combo_box_sizes_category.GetSize()[1])
        )
        self.spin_ctrl_sizes_input.SetMinSize((50, 22))
        self.combo_box_line_widths_category.SetMinSize(
            (250, self.combo_box_line_widths_category.GetSize()[1])
        )
        self.spin_ctrl_line_widths_input.SetMinSize((50, 22))
        self.combo_box_line_styles_category.SetMinSize(
            (250, self.combo_box_line_styles_category.GetSize()[1])
        )
        self.combo_box_line_styles_selection.SetMinSize(
            (145, self.combo_box_line_styles_selection.GetSize()[1])
        )
        self.combo_box_alpha_category.SetMinSize(
            (250, self.combo_box_alpha_category.GetSize()[1])
        )
        self.spin_ctrl_alpha_input.SetMinSize((70, 22))

        self.spin_ctrl_alpha_input.SetIncrement(0.1)

        # Windows needs this done explicitly or the value will be an empty string
        self.combo_box_alpha_category.SetValue("IQR Alpha")
        self.combo_box_colors_category.SetValue("Plot Color")
        self.combo_box_line_styles_category.SetValue("DVH Line Dash Selection")
        self.combo_box_line_widths_category.SetValue(
            "DVH Line Width Selection"
        )
        self.combo_box_sizes_category.SetValue("Plot Axis Label Font Size")

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_ok_cancel = wx.BoxSizer(wx.HORIZONTAL)
        sizer_plot_options = wx.StaticBoxSizer(
            wx.StaticBox(self, wx.ID_ANY, "Plot Options"), wx.VERTICAL
        )
        sizer_dvh_options = wx.StaticBoxSizer(
            wx.StaticBox(self, wx.ID_ANY, "DVH Options"), wx.VERTICAL
        )
        sizer_dvh_bin_width = wx.BoxSizer(wx.HORIZONTAL)
        sizer_dvh_bin_max = wx.BoxSizer(wx.HORIZONTAL)
        sizer_dvh_small_vol = wx.BoxSizer(wx.HORIZONTAL)
        sizer_dvh_segments_between = wx.BoxSizer(wx.HORIZONTAL)
        sizer_dvh_high_resolution = wx.BoxSizer(wx.HORIZONTAL)
        sizer_alpha = wx.BoxSizer(wx.VERTICAL)
        sizer_alpha_input = wx.BoxSizer(wx.HORIZONTAL)
        sizer_line_styles = wx.BoxSizer(wx.VERTICAL)
        sizer_line_styles_input = wx.BoxSizer(wx.HORIZONTAL)
        sizer_line_widths = wx.BoxSizer(wx.VERTICAL)
        sizer_line_widths_input = wx.BoxSizer(wx.HORIZONTAL)
        sizer_sizes = wx.BoxSizer(wx.VERTICAL)
        sizer_sizes_input = wx.BoxSizer(wx.HORIZONTAL)
        sizer_colors = wx.BoxSizer(wx.VERTICAL)
        sizer_colors_input = wx.BoxSizer(wx.HORIZONTAL)
        sizer_dicom_directories = wx.StaticBoxSizer(
            wx.StaticBox(self, wx.ID_ANY, "DICOM Directories"), wx.VERTICAL
        )
        sizer_imported_wrapper = wx.BoxSizer(wx.HORIZONTAL)
        sizer_imported = wx.BoxSizer(wx.VERTICAL)
        sizer_imported_input = wx.BoxSizer(wx.HORIZONTAL)
        sizer_inbox_wrapper = wx.BoxSizer(wx.HORIZONTAL)
        sizer_inbox = wx.BoxSizer(wx.VERTICAL)
        sizer_inbox_input = wx.BoxSizer(wx.HORIZONTAL)

        label_inbox = wx.StaticText(self, wx.ID_ANY, "Inbox:")
        label_inbox.SetToolTip(
            "Default directory for batch processing of incoming DICOM files"
        )
        sizer_inbox.Add(label_inbox, 0, 0, 5)
        sizer_inbox_input.Add(self.text_ctrl_inbox, 1, wx.ALL, 5)
        sizer_inbox_input.Add(self.button_inbox, 0, wx.ALL, 5)
        sizer_inbox.Add(sizer_inbox_input, 1, wx.EXPAND, 0)
        sizer_inbox_wrapper.Add(sizer_inbox, 1, wx.EXPAND, 0)
        sizer_dicom_directories.Add(sizer_inbox_wrapper, 1, wx.EXPAND, 0)

        label_imported = wx.StaticText(self, wx.ID_ANY, "Imported:")
        label_imported.SetToolTip("Directory for post-processed DICOM files")
        sizer_imported.Add(label_imported, 0, 0, 5)
        sizer_imported_input.Add(self.text_ctrl_imported, 1, wx.ALL, 5)
        sizer_imported_input.Add(self.button_imported, 0, wx.ALL, 5)
        sizer_imported.Add(sizer_imported_input, 1, wx.EXPAND, 0)
        sizer_imported_wrapper.Add(sizer_imported, 1, wx.EXPAND, 0)
        sizer_dicom_directories.Add(sizer_imported_wrapper, 1, wx.EXPAND, 0)
        sizer_wrapper.Add(sizer_dicom_directories, 0, wx.ALL | wx.EXPAND, 10)

        sizer_dvh_options.Add(self.checkbox_dicom_dvh, 0, wx.LEFT, 5)
        sizer_dvh_options.Add((20, 10), 0, 0, 0)
        label_dvh_bin_width = wx.StaticText(
            self, wx.ID_ANY, "DVH Bin Width (cGy):"
        )
        label_dvh_bin_width.SetToolTip("Value must be an integer")
        sizer_dvh_bin_width.Add(
            label_dvh_bin_width, 1, wx.EXPAND | wx.TOP | wx.LEFT, 5
        )
        sizer_dvh_bin_width.Add(self.dvh_bin_width_input, 0, wx.ALL, 5)
        label_max_dose_bin = wx.StaticText(
            self, wx.ID_ANY, "Max Dose Bin Limit:"
        )
        label_max_dose_bin.SetToolTip(
            "Prevent memory issues if dose grid contains very large doses"
        )
        sizer_dvh_bin_max.Add(
            label_max_dose_bin, 1, wx.EXPAND | wx.TOP | wx.LEFT, 5
        )
        sizer_dvh_bin_max.Add((20, 20), 0, 0, 0)
        sizer_dvh_bin_max.Add(self.dvh_bin_max_dose, 0, 0, 0)
        sizer_dvh_bin_max.Add((20, 20), 0, 0, 0)
        sizer_dvh_bin_max.Add(self.dvh_bin_max_dose_units, 0, 0, 0)
        sizer_dvh_options.Add(sizer_dvh_bin_width, 0, wx.EXPAND | wx.BOTTOM, 0)
        sizer_dvh_options.Add(sizer_dvh_bin_max, 0, wx.EXPAND, 0)

        label_dvh_small_volume = wx.StaticText(
            self, wx.ID_ANY, "Small volume threshold (cc):"
        )
        label_dvh_small_volume.SetToolTip(
            "If ROI volume is less than this value, it will be recalculated with "
            "a higher resolution using interpolation"
        )
        sizer_dvh_small_vol.Add(
            label_dvh_small_volume, 1, wx.EXPAND | wx.TOP, 5
        )
        sizer_dvh_small_vol.Add(self.dvh_small_volume_threshold, 0, wx.TOP, 5)
        sizer_dvh_options.Add(
            sizer_dvh_small_vol, 0, wx.EXPAND | wx.TOP | wx.LEFT, 5
        )

        label_dvh_segments_between = wx.StaticText(
            self, wx.ID_ANY, "Interpolated segments between planes:"
        )
        label_dvh_segments_between.SetToolTip(
            "If ROI volume is less than threshold, it will be recalculated with a this many "
            "segments interpolated between slices"
        )
        sizer_dvh_segments_between.Add(
            label_dvh_segments_between, 1, wx.EXPAND | wx.TOP, 5
        )
        sizer_dvh_segments_between.Add(self.dvh_segments_between, 0, wx.TOP, 5)
        sizer_dvh_options.Add(
            sizer_dvh_segments_between, 0, wx.EXPAND | wx.TOP | wx.LEFT, 5
        )

        label_dvh_high_resolution = wx.StaticText(
            self, wx.ID_ANY, "High resolution interpolation factor :"
        )
        label_dvh_high_resolution.SetToolTip(
            "If ROI volume is less than the volume threshold, the in-plane resolution will be "
            "increased by this factor (e.g., interpolate in-between the dose grid)"
        )
        sizer_dvh_high_resolution.Add(
            label_dvh_high_resolution, 1, wx.EXPAND | wx.TOP, 5
        )
        sizer_dvh_high_resolution.Add(self.dvh_high_resolution, 0, wx.TOP, 5)
        sizer_dvh_options.Add(
            sizer_dvh_high_resolution, 0, wx.EXPAND | wx.TOP | wx.LEFT, 5
        )

        sizer_wrapper.Add(sizer_dvh_options, 0, wx.ALL | wx.EXPAND, 10)

        label_colors = wx.StaticText(self, wx.ID_ANY, "Colors:")
        sizer_colors.Add(label_colors, 0, 0, 0)
        sizer_colors_input.Add(self.combo_box_colors_category, 0, 0, 0)
        sizer_colors_input.Add((20, 20), 0, 0, 0)
        sizer_colors_input.Add(self.combo_box_colors_selection, 0, 0, 0)
        sizer_colors.Add(sizer_colors_input, 1, wx.EXPAND, 0)
        sizer_plot_options.Add(sizer_colors, 1, wx.EXPAND, 0)

        label_sizes = wx.StaticText(self, wx.ID_ANY, "Sizes:")
        sizer_sizes.Add(label_sizes, 0, 0, 0)
        sizer_sizes_input.Add(self.combo_box_sizes_category, 0, 0, 0)
        sizer_sizes_input.Add((20, 20), 0, 0, 0)
        sizer_sizes_input.Add(self.spin_ctrl_sizes_input, 0, 0, 0)
        sizer_sizes.Add(sizer_sizes_input, 1, wx.EXPAND, 0)
        sizer_plot_options.Add(sizer_sizes, 1, wx.EXPAND, 0)

        label_line_widths = wx.StaticText(self, wx.ID_ANY, "Line Widths:")
        sizer_line_widths.Add(label_line_widths, 0, 0, 0)
        sizer_line_widths_input.Add(
            self.combo_box_line_widths_category, 0, 0, 0
        )
        sizer_line_widths_input.Add((20, 20), 0, 0, 0)
        sizer_line_widths_input.Add(self.spin_ctrl_line_widths_input, 0, 0, 0)
        sizer_line_widths.Add(sizer_line_widths_input, 1, wx.EXPAND, 0)
        sizer_plot_options.Add(sizer_line_widths, 1, wx.EXPAND, 0)

        label_line_styles = wx.StaticText(self, wx.ID_ANY, "Line Styles:")
        sizer_line_styles.Add(label_line_styles, 0, 0, 0)
        sizer_line_styles_input.Add(
            self.combo_box_line_styles_category, 0, 0, 0
        )
        sizer_line_styles_input.Add((20, 20), 0, 0, 0)
        sizer_line_styles_input.Add(
            self.combo_box_line_styles_selection, 0, 0, 0
        )
        sizer_line_styles.Add(sizer_line_styles_input, 1, wx.EXPAND, 0)
        sizer_plot_options.Add(sizer_line_styles, 1, wx.EXPAND, 0)

        label_alpha = wx.StaticText(self, wx.ID_ANY, "Alpha:")
        sizer_alpha.Add(label_alpha, 0, 0, 0)
        sizer_alpha_input.Add(self.combo_box_alpha_category, 0, 0, 0)
        sizer_alpha_input.Add((20, 20), 0, 0, 0)
        sizer_alpha_input.Add(self.spin_ctrl_alpha_input, 0, 0, 0)
        sizer_alpha.Add(sizer_alpha_input, 1, wx.EXPAND, 0)
        sizer_plot_options.Add(sizer_alpha, 1, wx.EXPAND, 0)
        sizer_wrapper.Add(
            sizer_plot_options, 1, wx.EXPAND | wx.LEFT | wx.RIGHT | wx.TOP, 10
        )

        sizer_ok_cancel.Add(self.button_restore_defaults, 0, wx.RIGHT, 20)
        sizer_ok_cancel.Add(self.button_apply, 0, wx.LEFT | wx.RIGHT, 5)
        sizer_ok_cancel.Add(self.button_ok, 0, wx.LEFT | wx.RIGHT, 5)
        sizer_ok_cancel.Add(self.button_cancel, 0, wx.LEFT | wx.RIGHT, 5)
        sizer_wrapper.Add(sizer_ok_cancel, 0, wx.ALIGN_RIGHT | wx.ALL, 10)

        self.SetSizer(sizer_wrapper)
        self.Layout()
        self.Fit()

        self.options.apply_window_position(self, "user_settings")

    def __do_bind(self):
        self.Bind(
            wx.EVT_BUTTON, self.inbox_dir_dlg, id=self.button_inbox.GetId()
        )
        self.Bind(
            wx.EVT_BUTTON,
            self.imported_dir_dlg,
            id=self.button_imported.GetId(),
        )

        self.Bind(
            wx.EVT_CHECKBOX,
            self.on_use_dicom_dvh,
            id=self.checkbox_dicom_dvh.GetId(),
        )

        self.Bind(
            wx.EVT_TEXT,
            self.update_dvh_bin_width_val,
            id=self.dvh_bin_width_input.GetId(),
        )
        self.Bind(
            wx.EVT_TEXT,
            self.update_dvh_bin_max_dose_val,
            id=self.dvh_bin_max_dose.GetId(),
        )
        self.Bind(
            wx.EVT_COMBOBOX,
            self.update_dvh_bin_max_dose_units_val,
            id=self.dvh_bin_max_dose_units.GetId(),
        )
        self.Bind(
            wx.EVT_TEXT,
            self.update_dvh_small_volume_val,
            id=self.dvh_small_volume_threshold.GetId(),
        )
        self.Bind(
            wx.EVT_TEXT,
            self.update_dvh_segments_between,
            id=self.dvh_segments_between.GetId(),
        )
        self.Bind(
            wx.EVT_COMBOBOX,
            self.update_dvh_high_resolution_factor,
            id=self.dvh_high_resolution.GetId(),
        )
        self.Bind(
            wx.EVT_COMBOBOX,
            self.update_input_colors_var,
            id=self.combo_box_colors_category.GetId(),
        )
        self.Bind(
            wx.EVT_COMBOBOX,
            self.update_size_var,
            id=self.combo_box_sizes_category.GetId(),
        )
        self.Bind(
            wx.EVT_COMBOBOX,
            self.update_line_width_var,
            id=self.combo_box_line_widths_category.GetId(),
        )
        self.Bind(
            wx.EVT_COMBOBOX,
            self.update_line_style_var,
            id=self.combo_box_line_styles_category.GetId(),
        )
        self.Bind(
            wx.EVT_COMBOBOX,
            self.update_alpha_var,
            id=self.combo_box_alpha_category.GetId(),
        )

        self.Bind(
            wx.EVT_COMBOBOX,
            self.update_input_colors_val,
            id=self.combo_box_colors_selection.GetId(),
        )
        self.Bind(
            wx.EVT_TEXT,
            self.update_size_val,
            id=self.spin_ctrl_sizes_input.GetId(),
        )
        self.Bind(
            wx.EVT_TEXT,
            self.update_line_width_val,
            id=self.spin_ctrl_line_widths_input.GetId(),
        )
        self.Bind(
            wx.EVT_COMBOBOX,
            self.update_line_style_val,
            id=self.combo_box_line_styles_selection.GetId(),
        )
        self.Bind(
            wx.EVT_TEXT,
            self.update_alpha_val,
            id=self.spin_ctrl_alpha_input.GetId(),
        )

        self.Bind(
            wx.EVT_BUTTON,
            self.restore_defaults,
            id=self.button_restore_defaults.GetId(),
        )
        self.Bind(wx.EVT_BUTTON, self.on_apply, id=self.button_apply.GetId())
        self.Bind(wx.EVT_BUTTON, self.on_ok, id=wx.ID_OK)
        self.Bind(wx.EVT_BUTTON, self.on_cancel, id=wx.ID_CANCEL)

        self.Bind(wx.EVT_CLOSE, self.on_cancel)

    def on_ok(self, *evt):
        if self.options.is_edited:  # Tracks edits since last redraw
            self.apply_and_redraw_plots()
        self.close()

    def on_cancel(self, *evt):
        self.options.load()
        if self.is_edited:  # Tracks edits since last options save
            self.apply_and_redraw_plots()
        self.close()

    def close(self, *evt):
        self.save_window_position()
        self.options.save()
        self.parent.user_settings = None
        self.Destroy()

    def save_window_position(self):
        self.options.save_window_position(self, "user_settings")

    def inbox_dir_dlg(self, evt):
        self.dir_dlg("inbox", self.text_ctrl_inbox)

    def imported_dir_dlg(self, evt):
        self.dir_dlg("imported", self.text_ctrl_imported)

    def dir_dlg(self, dir_type, text_ctrl):
        """
        Create a DirDialog and edit the associated TextCtrl
        :param dir_type: the directory type to be displayed in DirDialog title
        :type dir_type: str
        :param text_ctrl: the associated TextCtrl from the user settings dialog
        :type text_ctrl: TextCtrl
        """
        starting_dir = text_ctrl.GetValue()
        if not isdir(starting_dir):
            starting_dir = ""
        dlg = wx.DirDialog(
            self,
            "Select %s directory" % dir_type,
            starting_dir,
            wx.DD_DEFAULT_STYLE | wx.DD_DIR_MUST_EXIST,
        )
        if dlg.ShowModal() == wx.ID_OK:
            text_ctrl.SetValue(dlg.GetPath())
            option_attr = (
                "INBOX_DIR" if dir_type == "inbox" else "IMPORTED_DIR"
            )
            self.options.set_option(option_attr, dlg.GetPath())
        dlg.Destroy()

    def get_option_choices(self, category):
        """
        Lookup properties in Options.option_attr that fit the specified category
        :param category: COLOR, SIZE, ALPHA, LINE_WIDTH, LINE_DASH
        :type category: str
        :return: all options with the category in their name
        :rtype: list
        """
        choices = [
            self.clean_option_variable(c)
            for c in self.options.option_attr
            if c.find(category) > -1
        ]
        choices.sort()
        return choices

    @staticmethod
    def clean_option_variable(option_variable, inverse=False):
        """
        Convert option variable between UI and python format
        :param option_variable: option available for edit user settings UI
        :type option_variable: str
        :param inverse: True to return python format, False to return UI format
        :type inverse: bool
        :return: formatted option variable
        :rtype: str
        """
        if inverse:
            return option_variable.upper().replace(" ", "_")
        else:
            return (
                option_variable.replace("_", " ")
                .title()
                .replace("Dvh", "DVH")
                .replace("Iqr", "IQR")
            )

    def update_input_colors_var(self, *args):
        var = self.clean_option_variable(
            self.combo_box_colors_category.GetValue(), inverse=True
        )
        val = getattr(self.options, var)
        self.combo_box_colors_selection.SetValue(val)

    def update_input_colors_val(self, *args):
        var = self.clean_option_variable(
            self.combo_box_colors_category.GetValue(), inverse=True
        )
        val = self.combo_box_colors_selection.GetValue()
        self.options.set_option(var, val)

    def update_size_var(self, *args):
        var = self.clean_option_variable(
            self.combo_box_sizes_category.GetValue(), inverse=True
        )
        try:
            val = getattr(self.options, var).replace("pt", "")
        except AttributeError:
            val = str(getattr(self.options, var))
        try:
            val = int(float(val))
        except ValueError:
            pass
        self.spin_ctrl_sizes_input.SetValue(val)

    def update_size_val(self, *args):
        new = self.spin_ctrl_sizes_input.GetValue()
        if "Font" in self.combo_box_sizes_category.GetValue():
            try:
                val = str(int(new)) + "pt"
            except ValueError:
                val = "10pt"
        else:
            try:
                val = float(new)
            except ValueError:
                val = 1.0

        var = self.clean_option_variable(
            self.combo_box_sizes_category.GetValue(), inverse=True
        )
        self.options.set_option(var, val)

    def update_line_width_var(self, *args):
        var = self.clean_option_variable(
            self.combo_box_line_widths_category.GetValue(), inverse=True
        )
        val = str(getattr(self.options, var))
        try:
            val = int(float(val))
        except ValueError:
            pass
        self.spin_ctrl_line_widths_input.SetValue(val)

    def update_line_width_val(self, *args):
        new = self.spin_ctrl_line_widths_input.GetValue()
        try:
            val = int(float(new))
        except ValueError:
            val = 1
        var = self.clean_option_variable(
            self.combo_box_line_widths_category.GetValue(), inverse=True
        )
        self.options.set_option(var, val)

    def update_line_style_var(self, *args):
        var = self.clean_option_variable(
            self.combo_box_line_styles_category.GetValue(), inverse=True
        )
        self.combo_box_line_styles_selection.SetValue(
            getattr(self.options, var)
        )

    def update_line_style_val(self, *args):
        var = self.clean_option_variable(
            self.combo_box_line_styles_category.GetValue(), inverse=True
        )
        val = self.combo_box_line_styles_selection.GetValue()
        self.options.set_option(var, val)

    def update_alpha_var(self, *args):
        var = self.clean_option_variable(
            self.combo_box_alpha_category.GetValue(), inverse=True
        )
        self.spin_ctrl_alpha_input.SetValue(str(getattr(self.options, var)))

    def update_alpha_val(self, *args):
        new = self.spin_ctrl_alpha_input.GetValue()
        try:
            val = float(new)
        except ValueError:
            val = 1.0
        var = self.clean_option_variable(
            self.combo_box_alpha_category.GetValue(), inverse=True
        )
        self.options.set_option(var, val)

    def update_dvh_bin_width_val(self, *args):
        new = self.dvh_bin_width_input.GetValue()
        try:
            val = abs(int(new))
            self.options.set_option("dvh_bin_width", val)
        except ValueError:
            if new != "":
                self.dvh_bin_width_input.SetValue(
                    str(self.options.dvh_bin_width)
                )

    def update_dvh_bin_width_var(self, *args):
        self.dvh_bin_width_input.SetValue(str(self.options.dvh_bin_width))

    def update_dvh_bin_max_dose_val(self, *args):
        new_val = self.dvh_bin_max_dose.GetValue()
        units = self.dvh_bin_max_dose_units.GetValue()
        try:
            val = abs(float(new_val))
            new = {
                key: value
                for key, value in self.options.dvh_bin_max_dose.items()
            }
            new[units] = val
            self.options.set_option("dvh_bin_max_dose", new)
        except ValueError:
            if new_val != "":
                self.dvh_bin_max_dose.SetValue(
                    str(self.options.dvh_bin_max_dose[units])
                )

    def update_dvh_bin_max_dose_var(self, *args):
        units = self.dvh_bin_max_dose_units.GetValue()
        self.dvh_bin_max_dose.SetValue(
            str(self.options.dvh_bin_max_dose[units])
        )

    def update_dvh_bin_max_dose_units_val(self, *args):
        new = self.dvh_bin_max_dose_units.GetValue()
        self.options.set_option("dvh_bin_max_dose_units", new)
        self.dvh_bin_max_dose.SetValue(str(self.options.dvh_bin_max_dose[new]))

    def update_dvh_bin_max_dose_units_var(self, *args):
        self.dvh_bin_max_dose_units.SetValue(
            self.options.dvh_bin_max_dose_units
        )

    def update_dvh_small_volume_val(self, *args):
        try:
            new = int(float(self.dvh_small_volume_threshold.GetValue()))
            self.options.set_option("DVH_SMALL_VOLUME_THRESHOLD", new)
        except ValueError:
            self.dvh_small_volume_threshold.SetValue(
                str(self.options.DVH_SMALL_VOLUME_THRESHOLD)
            )

    def update_dvh_segments_between(self, *args):
        try:
            new = int(float(self.dvh_segments_between.GetValue()))
            self.options.set_option(
                "DVH_HIGH_RESOLUTION_SEGMENTS_BETWEEN", new
            )
        except ValueError:
            self.dvh_small_volume_threshold.SetValue(
                str(self.options.DVH_HIGH_RESOLUTION_SEGMENTS_BETWEEN)
            )

    def update_dvh_high_resolution_factor(self, *args):
        new = int(float(self.dvh_high_resolution.GetValue()))
        self.options.set_option("DVH_HIGH_RESOLUTION_FACTOR", new)

    def refresh_options(self):
        self.update_dvh_bin_width_var()
        self.update_dvh_bin_max_dose_var()
        self.update_dvh_bin_max_dose_units_var()
        self.update_alpha_var()
        self.update_input_colors_var()
        self.update_line_style_var()
        self.update_line_width_var()
        self.update_size_var()

    def load_paths(self):
        self.text_ctrl_inbox.SetValue(self.options.INBOX_DIR)
        self.text_ctrl_imported.SetValue(self.options.IMPORTED_DIR)

    def restore_defaults(self, *args):
        MessageDialog(
            self,
            "Restore default preferences?",
            action_yes_func=self.options.restore_defaults,
        )
        self.update_size_val()
        self.refresh_options()
        self.on_apply()

    def on_use_dicom_dvh(self, *evt):
        self.options.set_option(
            "USE_DICOM_DVH", self.checkbox_dicom_dvh.GetValue()
        )

    def on_apply(self, *evt):
        self.apply_and_redraw_plots()
        self.is_edited = True  # Used to track edits since last options save
        self.options.is_edited = False  # Used to track edits since redraw, is set to True on options.set_option()

    def apply_and_redraw_plots(self):
        self.parent.apply_plot_options()
        self.parent.redraw_plots()


class About(wx.Dialog):
    """
    Simple dialog to display the LICENSE file and a brief text header in a scrollable window
    """

    def __init__(self, *evt):
        wx.Dialog.__init__(self, None, title="About DVH Analytics")

        scrolled_window = wx.ScrolledWindow(self, wx.ID_ANY)

        with open(LICENSE_PATH, "r", encoding="utf8") as license_file:
            license_text = "".join([line for line in license_file])

        license_text = "DVH Analytics v%s\ndvhanalytics.com\n\n%s" % (
            DefaultOptions().VERSION,
            license_text,
        )

        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_text = wx.BoxSizer(wx.VERTICAL)

        scrolled_window.SetScrollRate(20, 20)

        license_text = wx.StaticText(scrolled_window, wx.ID_ANY, license_text)
        sizer_text.Add(license_text, 0, wx.EXPAND | wx.ALL, 5)
        scrolled_window.SetSizer(sizer_text)
        sizer_wrapper.Add(scrolled_window, 1, wx.EXPAND, 0)

        self.SetBackgroundColour(wx.WHITE)

        self.SetSizer(sizer_wrapper)
        self.SetSize((750, 900))
        self.Center()

        self.ShowModal()
        self.Destroy()


class PythonLibraries(wx.Dialog):
    """Simple dialog to display the installed python libraries"""

    def __init__(self, *evt):
        wx.Dialog.__init__(self, None, title="Installed Python Libraries")

        self.list_ctrl = wx.ListCtrl(
            self,
            wx.ID_ANY,
            style=wx.BORDER_SUNKEN
            | wx.LC_HRULES
            | wx.LC_REPORT
            | wx.LC_VRULES,
        )
        self.data_table = DataTable(self.list_ctrl, widths=[200, 150])

        self.__set_data()
        self.__do_layout()

        self.run()

    def __set_data(self):
        columns = ["Library", "Version"]
        try:
            libraries = get_installed_python_libraries()
        except Exception:
            libraries = {"Library": ["pip list failed"], "Version": [" "]}
        self.data_table.set_data(libraries, columns)

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_main = wx.BoxSizer(wx.VERTICAL)

        note = wx.StaticText(
            self,
            wx.ID_ANY,
            "NOTE: If running from source, this might reflect global packages.\n"
            "If running from an executable, this should be accurate.",
        )
        note.Wrap(500)

        sizer_main.Add(note, 0, wx.EXPAND | wx.ALL, 5)
        sizer_main.Add(self.list_ctrl, 1, wx.EXPAND, 0)
        sizer_wrapper.Add(sizer_main, 1, wx.EXPAND | wx.ALL, 10)

        self.SetBackgroundColour(wx.WHITE)
        self.SetSizer(sizer_wrapper)
        self.SetSize(get_window_size(0.3, 0.8))
        self.Center()

    def run(self):
        self.ShowModal()
        self.Destroy()


class ShowList(wx.Dialog):
    """Simple dialog to display a non-interactive List"""

    def __init__(self, list_items, title="List of Items", *evt):
        wx.Dialog.__init__(self, None, title=title)

        self.list_ctrl = wx.ListCtrl(
            self,
            wx.ID_ANY,
            style=wx.BORDER_SUNKEN
            | wx.LC_REPORT
            | wx.LC_NO_HEADER
            | wx.LC_SINGLE_SEL,
        )
        self.data_table = DataTable(self.list_ctrl, widths=[400])
        self.list_items = list_items
        self.data_table.set_data({"col": self.list_items}, ["col"])

        self.data_table.set_column_widths(auto=True)

        self.__do_bind()
        self.__do_layout()

        self.run()

    def __do_bind(self):
        self.Bind(
            wx.EVT_LIST_ITEM_SELECTED,
            self.on_select,
            id=self.list_ctrl.GetId(),
        )

    def __do_layout(self):
        sizer_wrapper = wx.BoxSizer(wx.VERTICAL)
        sizer_main = wx.BoxSizer(wx.VERTICAL)
        sizer_main.Add(self.list_ctrl, 1, wx.EXPAND, 0)
        sizer_wrapper.Add(sizer_main, 1, wx.EXPAND | wx.ALL, 10)

        self.SetBackgroundColour(wx.WHITE)
        self.SetSizer(sizer_wrapper)
        self.SetSize(get_window_size(0.25, 0.8))
        self.Center()

    def run(self):
        self.ShowModal()
        self.Destroy()

    def on_select(self, *evt):
        self.data_table.apply_selection_to_all(False)


def do_sqlite_backup(parent, options):
    flags = wx.ICON_ERROR | wx.OK | wx.OK_DEFAULT
    if options.DB_TYPE_GRPS[1] == "sqlite":
        try:
            file_paths = backup_sqlite_db(options)
            if file_paths is not None:
                msg = "Current DB: %s\nCopied to: %s" % (tuple(file_paths))
                caption = "SQLite DB Backup Successful"
                flags = wx.OK | wx.OK_DEFAULT
            else:
                msg = "Your SQLite DB was not backed up for an unknown reason."
                caption = "SQLite DB Backup Unsuccessful"
        except Exception as e:
            msg = str(e)
            caption = "SQLite DB Backup Error"
    else:
        msg = (
            "This feature only applies to SQLite users. PostgreSQL users should contact their DB "
            "administrator or look into the pg_dump command."
        )
        caption = "SQLite DB Backup Error"

    ErrorDialog(parent, msg, caption, flags=flags)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# models.rad_bio.py
"""
Class for the Rad Bio frame in the main view
"""
# Copyright (c) 2016-2019 Dan Cutright
# This file is part of DVH Analytics, released under a BSD license.
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics

import wx
from copy import deepcopy
from dvha.models.data_table import DataTable
from dvha.models.dvh import calc_eud, calc_tcp
from dvha.tools.stats import sync_variables_in_stats_data_objects
from dvha.tools.utilities import (
    convert_value_to_str,
    get_selected_listctrl_items,
    float_or_none,
)
from dvha.dialogs.export import save_data_to_file


class RadBioFrame:
    """
    Object to be passed into notebook panel for the Rad Bio tab
    """

    def __init__(self, main_app_frame):

        self.main_app_frame = main_app_frame
        self.parent = main_app_frame.notebook_tab["Rad Bio"]
        self.group_data = main_app_frame.group_data
        self.time_series = main_app_frame.time_series
        self.regression = main_app_frame.regression
        self.control_chart = main_app_frame.control_chart

        self.table_published_values = wx.ListCtrl(
            self.parent,
            wx.ID_ANY,
            style=wx.BORDER_SUNKEN
            | wx.LC_HRULES
            | wx.LC_REPORT
            | wx.LC_VRULES,
        )
        self.text_input_eud_a = wx.TextCtrl(self.parent, wx.ID_ANY, "")
        self.text_input_gamma_50 = wx.TextCtrl(self.parent, wx.ID_ANY, "")
        self.text_input_td_50 = wx.TextCtrl(self.parent, wx.ID_ANY, "")
        self.radio_box_query_group = wx.RadioBox(
            self.parent, wx.ID_ANY, "Query Group", choices=["1", "2", "Both"]
        )
        self.button_apply_parameters = wx.Button(
            self.parent, wx.ID_ANY, "Apply Parameters"
        )
        self.button_export = wx.Button(self.parent, wx.ID_ANY, "Export")
        self.table_rad_bio = {
            grp: wx.ListCtrl(
                self.parent,
                wx.ID_ANY,
                style=wx.BORDER_SUNKEN
                | wx.LC_HRULES
                | wx.LC_REPORT
                | wx.LC_VRULES,
            )
            for grp in [1, 2]
        }
        self.columns = [
            "MRN",
            "ROI Name",
            "a",
            u"\u03b3_50",
            "TD or TCD",
            "EUD",
            "NTCP or TCP",
            "PTV Overlap",
            "ROI Type",
            "Rx Dose",
            "Total Fxs",
            "Fx Dose",
        ]
        self.width = [100, 175, 50, 50, 80, 80, 80, 100, 100, 100, 100, 100]
        formats = [wx.LIST_FORMAT_RIGHT] * len(self.columns)
        formats[0] = wx.LIST_FORMAT_LEFT
        formats[1] = wx.LIST_FORMAT_LEFT
        self.data_table_rad_bio = {
            grp: DataTable(
                self.table_rad_bio[grp],
                columns=self.columns,
                widths=self.width,
                formats=formats,
            )
            for grp in [1, 2]
        }

        self.__set_properties()
        self.__do_layout()
        self.__do_bind()

        self.disable_buttons()

    def __set_properties(self):
        self.table_published_values.AppendColumn(
            "Structure", format=wx.LIST_FORMAT_LEFT, width=150
        )
        self.table_published_values.AppendColumn(
            "Endpoint", format=wx.LIST_FORMAT_LEFT, width=300
        )
        self.table_published_values.AppendColumn(
            "a", format=wx.LIST_FORMAT_LEFT, width=-1
        )
        self.table_published_values.AppendColumn(
            u"\u03b3_50", format=wx.LIST_FORMAT_LEFT, width=-1
        )
        self.table_published_values.AppendColumn(
            "TD_50", format=wx.LIST_FORMAT_LEFT, width=-1
        )

        for table in self.table_rad_bio.values():
            for i, col in enumerate(self.columns):
                table.AppendColumn(col, width=self.width[i])

        self.published_data = [
            ["Brain", "Necrosis", 5, 3, 60],
            ["Brainstem", "Necrosis", 7, 3, 65],
            ["Optic Chasm", "Blindness", 25, 3, 65],
            ["Colon", "Obstruction/Perforation", 6, 4, 55],
            ["Ear (mid/ext)", "Acute serous otitus", 31, 3, 40],
            ["Ear (mid/ext)", "Chronic serous otitus", 31, 4, 65],
            ["Esophagus", "Perforation", 19, 4, 68],
            ["Heart", "Pericarditus", 3, 3, 50],
            ["Kidney", "Nephritis", 1, 3, 28],
            ["Lens", "Cataract", 3, 1, 18],
            ["Liver", "Liver Failure", 3, 3, 40],
            ["Lung", "Pneumontis", 1, 2, 24.5],
            ["Optic Nerve", "Blindness", 25, 3, 65],
            ["Retina", "Blindness", 15, 2, 65],
        ]

        for row in self.published_data:
            index = self.table_published_values.InsertItem(50000, str(row[0]))
            for i in [1, 2, 3, 4]:
                self.table_published_values.SetItem(index, i, str(row[i]))

        self.radio_box_query_group.SetSelection(2)

    def __do_layout(self):
        sizer_main = wx.BoxSizer(wx.VERTICAL)
        sizer_parameters = wx.BoxSizer(wx.VERTICAL)
        sizer_parameters_input = wx.StaticBoxSizer(
            wx.StaticBox(self.parent, wx.ID_ANY, ""), wx.HORIZONTAL
        )
        sizer_buttons = wx.BoxSizer(wx.VERTICAL)
        sizer_td_50 = wx.BoxSizer(wx.VERTICAL)
        sizer_gamma_50 = wx.BoxSizer(wx.VERTICAL)
        sizer_eud = wx.BoxSizer(wx.VERTICAL)
        sizer_published_values = wx.BoxSizer(wx.VERTICAL)

        label_published_values = wx.StaticText(
            self.parent,
            wx.ID_ANY,
            "Published EUD Parameters from Emami et. al. for 1.8-2.0Gy "
            "fractions (Click to apply)",
        )
        label_published_values.SetFont(
            wx.Font(
                11,
                wx.FONTFAMILY_DEFAULT,
                wx.FONTSTYLE_NORMAL,
                wx.FONTWEIGHT_BOLD,
                0,
                "",
            )
        )
        sizer_published_values.Add(label_published_values, 0, wx.ALL, 5)
        sizer_published_values.Add(self.table_published_values, 0, wx.ALL, 5)
        sizer_main.Add(sizer_published_values, 0, wx.BOTTOM | wx.EXPAND, 5)

        label_parameters = wx.StaticText(self.parent, wx.ID_ANY, "Parameters:")
        label_parameters.SetFont(
            wx.Font(
                11,
                wx.FONTFAMILY_DEFAULT,
                wx.FONTSTYLE_NORMAL,
                wx.FONTWEIGHT_BOLD,
                0,
                "",
            )
        )
        sizer_parameters.Add(label_parameters, 0, 0, 0)

        label_eud = wx.StaticText(self.parent, wx.ID_ANY, "EUD a-value:")
        sizer_eud.Add(label_eud, 0, 0, 0)
        sizer_eud.Add(self.text_input_eud_a, 0, wx.ALL | wx.EXPAND, 5)
        sizer_parameters_input.Add(sizer_eud, 1, wx.EXPAND, 0)

        label_gamma_50 = wx.StaticText(self.parent, wx.ID_ANY, u"\u03b3_50:")
        sizer_gamma_50.Add(label_gamma_50, 0, 0, 0)
        sizer_gamma_50.Add(self.text_input_gamma_50, 0, wx.ALL | wx.EXPAND, 5)
        sizer_parameters_input.Add(sizer_gamma_50, 1, wx.EXPAND, 0)

        label_td_50 = wx.StaticText(self.parent, wx.ID_ANY, "TD_50 or TCD_50:")
        sizer_td_50.Add(label_td_50, 0, 0, 0)
        sizer_td_50.Add(self.text_input_td_50, 0, wx.ALL | wx.EXPAND, 5)
        sizer_parameters_input.Add(sizer_td_50, 1, wx.EXPAND, 0)

        sizer_parameters_input.Add(self.radio_box_query_group, 0, 0, 0)

        sizer_buttons.Add(
            self.button_apply_parameters, 0, wx.EXPAND | wx.LEFT | wx.RIGHT, 5
        )
        sizer_buttons.Add(
            self.button_export, 0, wx.EXPAND | wx.LEFT | wx.RIGHT, 5
        )
        sizer_parameters_input.Add(sizer_buttons, 1, wx.EXPAND, 0)
        sizer_parameters.Add(
            sizer_parameters_input, 1, wx.BOTTOM | wx.EXPAND, 5
        )
        sizer_main.Add(sizer_parameters, 0, wx.ALL | wx.EXPAND, 5)

        sizer_main.Add(
            wx.StaticText(self.parent, wx.ID_ANY, "Query Group 1:"),
            0,
            wx.BOTTOM,
            5,
        )
        sizer_main.Add(self.table_rad_bio[1], 0, wx.ALL | wx.EXPAND, 5)
        sizer_main.Add(
            wx.StaticText(self.parent, wx.ID_ANY, "Query Group 2:"),
            0,
            wx.BOTTOM,
            5,
        )
        sizer_main.Add(self.table_rad_bio[2], 0, wx.ALL | wx.EXPAND, 5)

        self.layout = sizer_main

    def __do_bind(self):
        self.parent.Bind(
            wx.EVT_LIST_ITEM_SELECTED,
            self.on_parameter_select,
            self.table_published_values,
        )
        self.parent.Bind(
            wx.EVT_BUTTON,
            self.apply_parameters,
            id=self.button_apply_parameters.GetId(),
        )
        self.parent.Bind(
            wx.EVT_BUTTON, self.on_export_csv, id=self.button_export.GetId()
        )

    def __set_tooltips(self):
        self.button_apply_parameters.SetToolTip(
            "Shift or Ctrl click for targeted application."
        )

    def enable_buttons(self):
        self.button_apply_parameters.Enable()

    def disable_buttons(self):
        self.button_apply_parameters.Disable()

    def enable_initial_buttons(self):
        self.button_apply_parameters.Enable()

    def on_parameter_select(self, evt):
        index = self.table_published_values.GetFirstSelected()

        self.text_input_eud_a.SetValue(str(self.published_data[index][2]))
        self.text_input_gamma_50.SetValue(str(self.published_data[index][3]))
        self.text_input_td_50.SetValue(str(self.published_data[index][4]))

    def update_dvh_data(self, group_data):
        """
        Import dvh data, store into self.dvh and set data in data_table
        :param group_data: group_data object from main frame
        :type group_data: dict
        """
        self.group_data = group_data
        for grp, data in group_data.items():
            dvh = data["dvh"]
            if dvh:
                data = {
                    "MRN": dvh.mrn,
                    "ROI Name": dvh.roi_name,
                    "a": [""] * dvh.count,
                    u"\u03b3_50": [""] * dvh.count,
                    "TD or TCD": [""] * dvh.count,
                    "EUD": [""] * dvh.count,
                    "NTCP or TCP": [""] * dvh.count,
                    "PTV Overlap": dvh.ptv_overlap,
                    "ROI Type": dvh.roi_type,
                    "Rx Dose": dvh.rx_dose,
                    "Total Fxs": dvh.total_fxs,
                    "Fx Dose": dvh.fx_dose,
                }
            else:
                data = {column: [] for column in self.columns}

            self.data_table_rad_bio[grp].set_data(data, self.columns)

    def apply_parameters(self, evt):
        """
        Calculate rad bio values based on parameters supplied by user,
        pass information on to other tabs in GUI
        """

        # Convert user supplied parameters from text to floats
        eud_a = float_or_none(self.text_input_eud_a.GetValue())
        gamma_50 = float_or_none(self.text_input_gamma_50.GetValue())
        td_50 = float_or_none(self.text_input_td_50.GetValue())

        groups = [[1], [2], [1, 2]][self.radio_box_query_group.GetSelection()]

        for grp in groups:
            data = self.group_data[grp]
            if data["dvh"]:
                # Get the indices of the selected rows, or assume all should be updated
                selected_indices = get_selected_listctrl_items(
                    self.table_rad_bio[grp]
                )
                if not selected_indices:
                    selected_indices = range(
                        self.data_table_rad_bio[grp].row_count
                    )

                # set the data in the datatable for the selected indices
                for i in selected_indices:
                    current_row = self.data_table_rad_bio[grp].get_row(i)
                    for j in [7, 9]:
                        current_row[j] = convert_value_to_str(current_row[j])
                    new_row = deepcopy(current_row)
                    new_row[2] = eud_a
                    new_row[3] = gamma_50
                    new_row[4] = td_50
                    try:
                        eud = calc_eud(
                            data["dvh"].dvh[:, i],
                            eud_a,
                            dvh_bin_width=data["dvh"].dvh_bin_width,
                        )
                        new_row[5] = "%0.2f" % round(eud, 2)
                    except Exception:
                        new_row[5] = "None"
                    try:
                        new_row[6] = "%0.2f" % round(
                            calc_tcp(gamma_50, td_50, float(new_row[5])), 3
                        )
                    except Exception:
                        new_row[6] = "None"
                    self.data_table_rad_bio[grp].edit_row(new_row, i)

                # Update data in in dvh object
                data["dvh"].eud = []
                data["dvh"].ntcp_or_tcp = []
                for i, eud in enumerate(
                    self.data_table_rad_bio[grp].data["EUD"]
                ):
                    data["dvh"].eud.append(float_or_none(eud))
                    data["dvh"].ntcp_or_tcp.append(
                        float_or_none(
                            self.data_table_rad_bio[grp].data["NTCP or TCP"][i]
                        )
                    )

                data["stats_data"].update_endpoints_and_radbio()
                if grp == 2:
                    sync_variables_in_stats_data_objects(
                        self.group_data[1]["stats_data"],
                        self.group_data[2]["stats_data"],
                    )

                # update data in time series
                self.time_series.update_y_axis_options()
                if self.time_series.combo_box_y_axis.GetValue() in [
                    "EUD",
                    "NTCP or TCP",
                ]:
                    self.time_series.update_plot()

                # update data in regression
                self.regression.update_combo_box_choices()

                # update data in control chart
                self.control_chart.update_combo_box_y_choices()
                if self.control_chart.combo_box_y_axis.GetValue() in [
                    "EUD",
                    "NTCP or TCP",
                ]:
                    self.control_chart.update_plot()

    def clear_data(self):
        for table in self.data_table_rad_bio.values():
            table.delete_all_rows()

    def get_csv(self, selection=None):
        csv = self.data_table_rad_bio[1].get_csv()
        if self.data_table_rad_bio[2].row_count:
            csv = "Group 1\n%s\n\nGroup 2\n%s" % (
                csv,
                self.data_table_rad_bio[2].get_csv(),
            )

        csv = csv.replace(
            "\u03b3", "gamma"
        )  # avoid UnicodeEncodeError when writing to file

        return csv

    def on_export_csv(self, evt):
        save_data_to_file(
            self.parent, "Export RadBio table to CSV", self.get_csv()
        )

    def get_save_data(self):
        return {
            group: data_table.get_save_data()
            for group, data_table in self.data_table_rad_bio.items()
        }

    def load_save_data(self, save_data):
        for grp in [1, 2]:
            self.data_table_rad_bio[grp].load_save_data(save_data[grp])

    @property
    def has_data(self):
        return any(self.data_table_rad_bio[1].data["a"])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# tools.roi_formatter.py
"""Formatting tools for roi data (dicompyler, Shapely, DVHA)"""
# Copyright (c) 2016-2019 Dan Cutright
# This file is part of DVH Analytics, released under a BSD license.
#    See the file LICENSE included with this distribution, also
#    available at https://github.com/cutright/DVH-Analytics

from shapely.geometry import Polygon, Point
from shapely import speedups
import numpy as np

# "sets of points" objects
#     dictionaries using str(z) as keys
#         where z is the slice or z in DICOM coordinates
#         each item is a list of points representing a polygon,
#         each point is a 3-item list [x, y, z]
#
# roi_coord_string from the SQL database
#     Each contour is delimited with a ':'
#         For example, ring ROIs will have an outer contour with a negative
#         inner contour
#     Each contour is a csv of of x,y,z values in the following format
#         z,x1,y1,x2,y2...xn,yn
#         Each contour has the same z coordinate for all points


MIN_SLICE_THICKNESS = 2  # Update method to pull from DICOM


# Enable shapely calculations using C, as opposed to the C++ default
if speedups.available:
    speedups.enable()


def get_planes_from_string(roi_coord_string):
    """

    Parameters
    ----------
    roi_coord_string : string: str
        roi string representation of an roi as formatted in the SQL database

    Returns
    -------
    dict
        a "sets of points" formatted dictionary

    """
    planes = {}
    contours = roi_coord_string.split(":")

    for contour in contours:
        contour = contour.split(",")
        z = contour.pop(0)
        z = round(float(z), 2)
        z_str = str(z)

        if z_str not in list(planes):
            planes[z_str] = []

        i, points = 0, []
        while i < len(contour):
            point = [float(contour[i]), float(contour[i + 1]), z]
            points.append(point)
            i += 2
        planes[z_str].append(points)

    return planes


def points_to_shapely_polygon(sets_of_points):
    """

    Parameters
    ----------
    sets_of_points : dict
        a "sets of points" formatted dictionary

    Returns
    -------
    type
        a composite polygon as a shapely object (either polygon or multipolygon)

    """

    composite_polygon = None
    for set_of_points in sets_of_points:
        if len(set_of_points) > 3:
            points = [(point[0], point[1]) for point in set_of_points]
            points.append(
                points[0]
            )  # Explicitly connect the final point to the first

            # if there are multiple sets of points in a slice, each set is a polygon,
            # interior polygons are subtractions, exterior are addition
            # Only need to check one point for interior vs exterior
            current_polygon = Polygon(points).buffer(0)  # clean stray points
            if composite_polygon:
                if Point((points[0][0], points[0][1])).disjoint(
                    composite_polygon
                ):
                    composite_polygon = composite_polygon.union(
                        current_polygon
                    )
                else:
                    composite_polygon = composite_polygon.symmetric_difference(
                        current_polygon
                    )
            else:
                composite_polygon = current_polygon

    return composite_polygon


def get_roi_coordinates_from_string(roi_coord_string):
    """

    Parameters
    ----------
    roi_coord_string : string: str
        roi string representation of an roi as formatted in the SQL database

    Returns
    -------
    list
        a list of numpy arrays, each array is the x, y, z coordinates of the given point

    """
    roi_coordinates = []
    contours = roi_coord_string.split(":")

    for contour in contours:
        contour = contour.split(",")
        z = contour.pop(0)
        z = float(z)
        i = 0
        while i < len(contour):
            roi_coordinates.append(
                np.array((float(contour[i]), float(contour[i + 1]), z))
            )
            i += 2

    return roi_coordinates


def get_roi_coordinates_from_planes(sets_of_points):
    """

    Parameters
    ----------
    sets_of_points : dict
        a "sets of points" formatted dictionary

    Returns
    -------
    list
        a list of numpy arrays, each array is the x, y, z coordinates of the given point

    """
    roi_coordinates = []

    for z in list(sets_of_points):
        for polygon in sets_of_points[z]:
            for point in polygon:
                roi_coordinates.append(
                    np.array((point[0], point[1], point[2]))
                )
    return roi_coordinates


def dicompyler_roi_coord_to_db_string(coord):
    """

    Parameters
    ----------
    coord :
        dicompyler structure coordinates from GetStructureCoordinates()

    Returns
    -------
    str
        roi string representation of an roi as formatted in the SQL database (roi_coord_string)

    """
    contours = []
    for z in coord:
        for plane in coord[z]:
            points = [z]
            for point in plane["data"]:
                points.append(str(round(point[0], 3)))
                points.append(str(round(point[1], 3)))
            contours.append(",".join(points))
    return ":".join(contours)


def get_shapely_from_sets_of_points(sets_of_points):
    """

    Parameters
    ----------
    sets_of_points : dict
        a "sets of points" formatted dictionary

    Returns
    -------
    list
        roi_slices which is a dictionary of lists of z, thickness, and a Shapely Polygon class object

    """

    roi_slices = {"z": [], "thickness": [], "polygon": []}

    sets_of_points_keys = list(sets_of_points)
    sets_of_points_keys.sort()

    all_z_values = [round(float(z), 2) for z in sets_of_points_keys]
    thicknesses = np.abs(np.diff(all_z_values))
    if len(thicknesses):
        thicknesses = np.append(thicknesses, np.min(thicknesses))
    else:
        thicknesses = np.array([MIN_SLICE_THICKNESS])

    for z in sets_of_points:
        thickness = thicknesses[all_z_values.index(round(float(z), 2))]
        shapely_roi = points_to_shapely_polygon(sets_of_points[z])
        if shapely_roi:
            roi_slices["z"].append(round(float(z), 2))
            roi_slices["thickness"].append(thickness)
            roi_slices["polygon"].append(shapely_roi)

    return roi_slices


def dicompyler_roi_to_sets_of_points(coord):
    """

    Parameters
    ----------
    coord :
        dicompyler structure coordinates from GetStructureCoordinates()

    Returns
    -------
    dict
        a "sets of points" formatted dictionary

    """
    all_points = {}
    for z in coord:
        all_points[z] = []
        for plane in coord[z]:
            plane_points = [
                [float(point[0]), float(point[1])] for point in plane["data"]
            ]
            for point in plane["data"]:
                plane_points.append([float(point[0]), float(point[1])])
            if len(plane_points) > 2:
                all_points[z].append(plane_points)
    return all_points
